import numpy as np

import SpectrumCommon.Const.GB.GBConst as gbc

import SpectrumCommon.Const.IS.ISConst as isc
import SpectrumCommon.Const.IS.ISConstLink as iscl
import SpectrumCommon.Const.IS.ISModVarFields as ismvf

#######################################################################################################
#                                                                                                     #
#                                       Facility type records                                         #
#                                                                                                     #
#######################################################################################################

#######################################################################################################
#                                                                                                     #
#                                       Facilities                                                    #
#                                                                                                     #
#######################################################################################################

def IS_create_facility_type_record(
    ID = str, name = str, string_const = str, active = bool, support_type_mst_ID = int, 
    deliv_chan_mst_ID = str, def_facility_type_map_mst_ID = int, num_years = int, costing_mode = int, plan_type = int, num_prog_areas = int
):

    facility_rec = {
        ismvf.IS_FACIL_TYPE_ID                                   : ID,
        ismvf.IS_FACIL_STR_CONSTANT                              : string_const,
        ismvf.IS_FACIL_NAME                                      : name,
        ismvf.IS_FACIL_ACTIVE                                    : active,
        ismvf.IS_FACIL_CUSTOM                                    : False,
        ismvf.IS_FACIL_SUPP_TYPE_MST_ID                          : support_type_mst_ID,
        ismvf.IS_FACIL_DELIV_CHAN_MST_ID                         : deliv_chan_mst_ID,
        ismvf.IS_FACIL_DEFAULT_MAP_MST_ID                        : def_facility_type_map_mst_ID,
        ismvf.IS_FACIL_TOTAL_NUMBER_BASELINE                     : 0,
        ismvf.IS_FACIL_AVG_NUM_BEDS                              : 0,
        ismvf.IS_FACIL_AVG_OCCUP_RATE_BEDS                       : 0,
        ismvf.IS_FACIL_AVG_NUM_OUTPAT_VISITS_PER_YEAR            : 0,
        ismvf.IS_FACIL_YEARS_TO_BUILD                            : 0,
        ismvf.IS_FACIL_TOTAL_CONSTRUCT_COST                      : 0,
        ismvf.IS_FACIL_LAND_COST                                 : 0.0,
        ismvf.IS_FACIL_CONSTRUCT_COST_PER_SQ_M                   : 0.0,
        ismvf.IS_FACIL_AVG_SIZE_IN_SQ_M                          : 0.0,
        ismvf.IS_FACIL_REHAB_COST                                : np.full((isc.IS_NUM_REHAB_TYPES), 0.0).tolist(),
        ismvf.IS_FACIL_TOTAL_OP_COST                             : 0.0,
        ismvf.IS_FACIL_TOTAL_OP_COSTS_PERC_CONSTRUCT_COST        : 0.0,
        ismvf.IS_FACIL_DESIRED_POP_FACIL_RATIO_POP_NORMS         : 0,
        ismvf.IS_FACIL_TARG_SCALE_UP_POP_NORMS                   : np.full((num_years), 0).tolist(),
        ismvf.IS_FACIL_TARG_SCALE_UP_EXIST_PLANS_EXPECTED_OP     : np.full((num_years), 0).tolist(),
        ismvf.IS_FACIL_TARG_SCALE_UP_EXIST_PLANS_BEGIN_CONSTRUCT : np.full((num_years), 0).tolist(),
        ismvf.IS_FACIL_TARG_NUM_TO_REHAB                         : np.full((isc.IS_NUM_REHAB_TYPES), 0.0).tolist(),
        ismvf.IS_FACIL_TARG_NUM_TO_REHAB_BY_YEAR                 : np.full((isc.IS_NUM_REHAB_TYPES, num_years), 0.0).tolist(),
    }

    # In IHT specific-plan mode, we need certain fields to vary by programme/health area
    if (costing_mode == gbc.GB_IHT_COSTING_MODE) and (plan_type == iscl.IS_IH_SPECIFIC_PLAN):
        facility_rec[ismvf.IS_FACIL_NUM_TO_EQUIP] = np.full((num_prog_areas, num_years), 0).tolist()
        facility_rec[ismvf.IS_FACIL_EQUIP_PURCHASE_COST] = np.full((num_prog_areas, isc.IS_NUM_FACIL_EQUIP_TYPES), 0.0).tolist()

    else: 
        facility_rec[ismvf.IS_FACIL_NUM_TO_EQUIP] = np.full((num_years), 0).tolist()
        facility_rec[ismvf.IS_FACIL_EQUIP_PURCHASE_COST] = np.full((isc.IS_NUM_FACIL_EQUIP_TYPES), 0.0).tolist()

    if costing_mode == gbc.GB_TB_COSTING_MODE:
        facility_rec[ismvf.IS_FACIL_MED_EQUIP_MAINT_COST] = 0
        facility_rec[ismvf.IS_FACIL_MED_EQUIP_CALIB_COST] = 0
    
    elif (costing_mode == gbc.GB_IHT_COSTING_MODE) and (plan_type == iscl.IS_IH_SPECIFIC_PLAN):
        facility_rec[ismvf.IS_FACIL_MED_EQUIP_MAINT_COST] = np.full((num_prog_areas), 0).tolist()
        facility_rec[ismvf.IS_FACIL_MED_EQUIP_CALIB_COST] = np.full((num_prog_areas), 0).tolist()

    return facility_rec

def IS_init_facility_type_records(num_years, costing_mode = int, plan_type = int, num_prog_areas = int):
    facility_type_records = []

    facility_type_records.append(
        IS_create_facility_type_record(
            str(isc.IS_HEALTH_POST_MST_ID), 
            'Health post', 
            'GB_stHealthPost', 
            True,
            isc.IS_FACILITY_DELIV_INTERV_MST_ID, 
            str(iscl.IS_IH_COMMUNITY_CHAN_MST_ID),
            isc.IS_HEALTH_POST_MST_ID,
            num_years,
            costing_mode,
            plan_type,
            num_prog_areas
        )
    )

    facility_type_records.append(
        IS_create_facility_type_record(
            str(isc.IS_PREHOSPITAL_EMERGENCY_MST_ID), 
            'Prehospital emergency', 
            'GB_stPrehospitalEmergency', 
            True,
            isc.IS_FACILITY_DELIV_INTERV_MST_ID, 
            str(iscl.IS_IH_PREHOSPITAL_EMERGENCY_CHAN_MST_ID),
            isc.IS_PREHOSPITAL_EMERGENCY_MST_ID,
            num_years,
            costing_mode,
            plan_type,
            num_prog_areas
        )
    )

    facility_type_records.append(
        IS_create_facility_type_record(
            str(isc.IS_HEALTH_CENTER_MST_ID), 
            'Health centre', 
            'GB_stHealthCentre', 
            True,
            isc.IS_FACILITY_DELIV_INTERV_MST_ID, 
            str(iscl.IS_IH_GEN_OUTPATIENT_SERV_CHAN_MST_ID),
            isc.IS_HEALTH_CENTER_MST_ID, 
            num_years,
            costing_mode,
            plan_type,
            num_prog_areas
        )
    )

    facility_type_records.append(
        IS_create_facility_type_record(
            str(isc.IS_FREE_STAND_GEN_OUTPAT_CLINIC_MST_ID), 
            'Free-standing general outpatient clinic', 
            'GB_stFreeStandingGenOutpatientClinic', 
            True,
            isc.IS_FACILITY_DELIV_INTERV_MST_ID, 
            str(iscl.IS_IH_GEN_OUTPATIENT_SERV_CHAN_MST_ID),
            isc.IS_FREE_STAND_GEN_OUTPAT_CLINIC_MST_ID,
            num_years,
            costing_mode,
            plan_type,
            num_prog_areas
        )
    )

    facility_type_records.append(
        IS_create_facility_type_record(
            str(isc.IS_DISTRICT_GEN_HOSPITAL_MST_ID), 
            'District/General hospital', 
            'GB_stDistrictGeneralHospital', 
            True,
            isc.IS_FACILITY_DELIV_INTERV_MST_ID, 
            str(iscl.IS_IH_FIRST_REFERRAL_CHAN_MST_ID),
            isc.IS_DISTRICT_GEN_HOSPITAL_MST_ID, 
            num_years,
            costing_mode,
            plan_type,
            num_prog_areas
        )
    )

    facility_type_records.append(
        IS_create_facility_type_record(
            str(isc.IS_NAT_REG_PROV_HOSPITAL_MST_ID), 
            'National/Regional/Provincial hospital', 
            'GB_stNatRegProvincialHospital', 
            True,
            isc.IS_FACILITY_DELIV_INTERV_MST_ID, 
            str(iscl.IS_IH_SEC_REFERRAL_CHAN_MST_ID),
            isc.IS_NAT_REG_PROV_HOSPITAL_MST_ID, 
            num_years,
            costing_mode,
            plan_type,
            num_prog_areas
        )
    )

    facility_type_records.append(
        IS_create_facility_type_record(
            str(isc.IS_FREE_STAND_SPEC_OUTPAT_CLINIC_MST_ID), 
            'Free-standing specialized outpatient clinic', 
            'GB_stFreeStandingSpecOutpatientClinic', 
            True,
            isc.IS_FACILITY_DELIV_INTERV_MST_ID, 
            str(iscl.IS_IH_SEC_REFERRAL_CHAN_MST_ID),
            isc.IS_FREE_STAND_SPEC_OUTPAT_CLINIC_MST_ID, 
            num_years,
            costing_mode,
            plan_type,
            num_prog_areas
        )
    )

    # facility_type_records.append(
    #     IS_create_facility_type_record(
    #         str(isc.IS_CENTRAL_HOSPITAL_MST_ID), 
    #         'Central Hospital', 
    #         'GB_stCentralHospital', 
    #         True,
    #         isc.IS_FACILITY_DELIV_INTERV_MST_ID, 
    #         str(iscl.IS_IH_FIRST_REFERRAL_CHAN_MST_ID),
    #         isc.IS_CENTRAL_HOSPITAL_MST_ID, 
    #         num_years,
    #        costing_mode  
    #     )
    # )

    return facility_type_records

def IS_create_vehicle_type_record(ID = str, name = str, num_facilities = int):

    return {
        ismvf.IS_VEHIC_ID                          : ID,
        ismvf.IS_VEHIC_NAME                        : name,
        ismvf.IS_VEHIC_CUSTOM                      : False,
        ismvf.IS_VEHIC_CURR_NUMBER                 : 0.0,
        ismvf.IS_VEHIC_UNIT_COST                   : 0.0,
        ismvf.IS_VEHIC_WORKING_LIFE_YEARS          : 5.0,
        ismvf.IS_VEHIC_DRIVER_SALARY               : 0.0,
        ismvf.IS_VEHIC_FUEL_CONSUMPTION            : 0.0,
        ismvf.IS_VEHIC_DISTANCE_DRIVEN_PER_YEAR    : 0.0,
        ismvf.IS_VEHIC_FUEL_COST_DIRECT_ENTRY      : 0.0,
        ismvf.IS_VEHIC_REPAIR_MAINT_PERC_CAP_COST  : 0.0,
        ismvf.IS_VEHIC_TARG_NUM_PER_FACILITY       : np.full((num_facilities), 0).tolist(),
        ismvf.IS_VEHIC_TARG_NUM_PER_FACILITY_OTHER : 0,
    }

def IS_create_ICT_equip_type_record(ID = str, name = str, num_facilities = int):

    return {
        ismvf.IS_ICT_EQUIP_ID                      : ID,
        ismvf.IS_ICT_EQUIP_NAME                    : name,
        ismvf.IS_ICT_EQUIP_CUSTOM                  : False,
        ismvf.IS_ICT_EQUIP_CURR_NUMBER             : 0.0,
        ismvf.IS_ICT_EQUIP_UNIT_COST               : 0.0,
        ismvf.IS_ICT_EQUIP_WORKING_LIFE_YEARS      : 0.0,
        ismvf.IS_ICT_EQUIP_OP_COST_PERC_CAP_COST   : 0.0,
        ismvf.IS_ICT_REPAIR_MAINT_PERC_CAP_COST    : 0.0,
        ismvf.IS_ICT_TARG_NUM_PER_FACILITY         : np.full((num_facilities), 0).tolist(),
        ismvf.IS_ICT_TARG_NUM_PER_FACILITY_OTHER   : 0,
    }

def IS_create_facility_MEG_record(ID = str, name = str, facility_type_ID = str):

    return {
        ismvf.IS_FACIL_MEG_ID            : ID,
        ismvf.IS_FACIL_MEG_NAME          : name,
        ismvf.IS_FACIL_EQUIP_STR_CONST   : '',
        ismvf.IS_FACIL_MEG_FACIL_TYPE_ID : facility_type_ID,
    }

# Use for both facility medical equipment, which goes by group, and furniture, which does not. They currently have the same fields, but if that
# ceases to be the case, copy what's happening with the three types of treatment inputs in Intervention Costing
def IS_create_facility_equip_record(ID = str, name = str, facility_type_ID = str, facility_equip_type_mst_ID = int, costing_mode = int, IHT_plan_type = int):
    
    equip_record = {
        ismvf.IS_FACIL_EQUIP_ID            : ID, 
        ismvf.IS_FACIL_EQUIP_CUSTOM        : False,
        ismvf.IS_FACIL_EQUIP_NAME          : name,
        ismvf.IS_FACIL_MEG_STR_CONST       : '',
        ismvf.IS_FACIL_EQUIP_FACIL_TYPE_ID : facility_type_ID,        
        ismvf.IS_FACIL_EQUIP_UNIT_COST     : 0.0,
        ismvf.IS_FACIL_EQUIP_NUM_UNITS     : 0.0,
        ismvf.IS_FACIL_EQUIP_WORKING_LIFE  : 5.0,
    }

    # Only add this field for medical equipment. Irrelevant for furniture. 
    if facility_equip_type_mst_ID == isc.IS_FACIL_MEDICAL_EQUIP_MST_ID:
        equip_record[ismvf.IS_FACIL_EQUIP_MEG_ID] = "0"

        if (costing_mode == gbc.GB_TB_COSTING_MODE) or ((costing_mode == gbc.GB_IHT_COSTING_MODE) and (IHT_plan_type == iscl.IS_IH_SPECIFIC_PLAN)):
            equip_record[ismvf.IS_FACIL_EQUIP_MAINT_COST] = 0
            equip_record[ismvf.IS_FACIL_EQUIP_CALIB_COST] = 0

    return equip_record

#######################################################################################################
#                                                                                                     #
#                                            Results                                                  #
#                                                                                                     #
#######################################################################################################

def IS_init_facility_year_arr(num_facility_types = int, num_years = int, value = 0.0):
    return np.full((num_facility_types, num_years), value, gbc.GB_CALC_FLOAT_TYPE)

def IS_init_facility_prog_area_year_arr(num_facility_types = int, num_prog_areas = int, num_years = int, value = 0.0):
    return np.full((num_facility_types, num_prog_areas, num_years), value, gbc.GB_CALC_FLOAT_TYPE)

def IS_init_vehicle_year_arr(num_vehicle_types = int, num_years = int, value = 0.0):
    return np.full((num_vehicle_types, num_years), value, gbc.GB_CALC_FLOAT_TYPE)

def IS_init_vehicle_facility_year_arr(num_vehicle_types = int, num_facility_types = int, num_years = int, value = 0.0):
    return np.full((num_vehicle_types, num_facility_types, num_years), value, gbc.GB_CALC_FLOAT_TYPE)

def IS_init_ICT_year_arr(num_ICT_types = int, num_years = int, value = 0.0):
    return np.full((num_ICT_types, num_years), value, gbc.GB_CALC_FLOAT_TYPE)

def IS_init_ICT_facility_year_arr(num_ICT_types = int, num_facility_types = int, num_years = int, value = 0.0):
    return np.full((num_ICT_types, num_facility_types, num_years), value, gbc.GB_CALC_FLOAT_TYPE)



from os import environ
import numpy as np

from AvenirCommon.Database import GB_get_db_json

from SpectrumCommon.Const.DP import DP_NaomiEstimateChoice, DP_NaomiFinalAgeCount, DP_BigPopTag, DP_MigrRateTag
from SpectrumCommon.Const.GB import GB_SPECT_MOD_DATA_CONN_ENV
from SpectrumCommon.Const.PJ.PJNTags import PJN_ISO3AlphaTag

def dp_apply_naomi_subnat(projection, subnat_region):

    
    subnat = GB_get_db_json(environ[GB_SPECT_MOD_DATA_CONN_ENV], 'demproj', f'subnationals\\{subnat_region}_V1.JSON')
    
    
    nat = GB_get_db_json(environ[GB_SPECT_MOD_DATA_CONN_ENV], 'demproj', f'subnationals\\{projection[PJN_ISO3AlphaTag]}_V1.JSON')
    ratio_2023 = np.array(subnat['population'][DP_NaomiEstimateChoice])[:, DP_NaomiFinalAgeCount].sum()/np.array(nat['population'][DP_NaomiEstimateChoice])[:, DP_NaomiFinalAgeCount].sum()
    projection['ratio_2023'] = ratio_2023
    
    # survRate = projection[DP_SurvRateTag]
    projection[DP_BigPopTag] = projection[DP_BigPopTag]*ratio_2023
    projection[DP_MigrRateTag] = projection[DP_MigrRateTag]*0.0
    
    # subnat_year_ratio = 
    
    # popFiveYearAgeGroups = np.zeros((18, 3))
    # # popFiveYearAgeGroups[1:18, 1:] = np.array(subnat['population'][DP_NaomiEstimateChoice])[:, :DP_NaomiFinalAgeCount].transpose()
    # popFiveYearAgeGroups[1:18, 1:] = (np.array(nat['population'][DP_NaomiEstimateChoice])[:, :DP_NaomiFinalAgeCount]*nat_ratio).transpose()
    # convertFiveYearAgeGroupPopsToSingleYearAgePops({DP_SurvRateTag:survRate, DP_BigPopTag:bigPop}, popFiveYearAgeGroups)

    return projection



#  HV_MAX_LINE = 80 #

# Read Write determines size of numbers in file #
HV_FileNumSpace = 20
HV_FileRDec = 5

# Error Messages #
HV_NOERROR           = 0
HV_ERROR             = 1

# Default constants #
HV_Default  = 1
HV_Data     = 2

# Used for CustomEasyData imported from projection #
HV_ImportedProj  = 1

# Space #
HV_Space = '   '

# Special constants #
HV_DataFileTitle     = 'HIV Vaccine data file  - The POLICY Project'
HV_Version10         = 1.0
HV_Version11         = 1.1
HV_Version12         = 1.2
HV_Version13         = 1.3
HV_Version14         = 1.4

# Sex constants #
HV_BothSexes = 0
HV_Male      = 1
HV_Female    = 2

# Indicator Constants #
HV_SEXIND            = 1
HV_SERV_IND          = 2
HV_SINGLE_SERV_IND   = 3
HV_Summary_IND       = 4
HV_DISPLAYVALUES_IND = 5
HV_NO_SCALE_TABLE    = 6
HV_CD4Distribution   = 7
HV_ONLY_ONE_PROJ     = 8
HV_METHODIND         = 9
HV_max_services      = 10

# Display Chart Types #
HV_DispLine2d   = 0
HV_DispBar2d    = 1
HV_DispBar3d    = 2
HV_DispHBar2d   = 3
HV_DispHBar3d   = 4
HV_DispTable    = 5

# Display constants #
HV_Number = 0
HV_Percent = 1

# Time constants #
HV_FIRST_INDEX   = 1
HV_min_year      = 1

# Time step #
HV_TimeStep = 10

# Age Constats #
HV_A15_19  = 1
HV_A20_24  = 2
HV_A25_29  = 3
HV_A30_34  = 4
HV_A35_39  = 5
HV_A40_44  = 6
HV_A45_49  = 7
HV_A50_54  = 8
HV_A55_59  = 9
HV_A60_64  = 10

# Impact Matrix constants #
HV_Average        = 0
HV_UpperQuartile  = 1
HV_LowerQuartile  = 2

# Projection constants#
HV_max_proj = 10
GB_FirstProj = 1

# Scale constants #
HV_units             = 1
HV_thousands         = 2
HV_millions          = 3
HV_thousand_millions = 4

HV_Table           = 101
HV_PyramidPer      = 102
HV_PyramidNum      = 103
HV_SummaryTable    = 104
HV_Stacked2D       = 105
HV_Stacked3D       = 106

# Regions #
HV_ALL_REGION         = 0
HV_URBAN              = 1
HV_RURAL              = 2

# Married #
HV_ALL_WOMEN          = 0
HV_MARRIED            = 1
HV_UNMARRIED          = 2

# Condom Use constants #
HV_UseLogisticsCurve     = 0
HV_UseInterpolatedValues = 1

# Should be set to the Max of either Edit Services or Edit Sectors or PrEP Methods#
#  HV_max_DisAg          = 23

# Variable Parameters #
HV_YEAR_DIM          = 1
HV_ALL_SERVICES      = 0

HV_MAX_EDIT_SEC      = 13
HV_MAXSEC            = 14
HV_ModelFittingSec   = 1
HV_PopSec            = 2
HV_NewInfectionsSec  = 3
HV_HIVPosSec         = 4
HV_ART               = 5
HV_DeathsSec         = 6
HV_VaccsSec          = 7
HV_PrEPSec           = 8
HV_BenefitsSec       = 9
HV_CostSec           = 10
HV_SummarySec        = 11
HV_GoalsVsAIMSec     = 12
HV_MultiplierSec     = 13
HV_InputsSec         = 14

#Indicators#
HV_cmNewInfections          = 1
HV_cmNumHIVPos              = 2
HV_cmAIDSDeaths             = 3
HV_cmNumVaccs               = 4
HV_cmNumVaccinated          = 5
HV_cmIncidence              = 6
HV_cmPrevalence             = 7
HV_cmPrevRG                 = 8
HV_cmNewInfectionsRG        = 9
HV_cmAIDSDeathsRG           = 10
HV_cmPopbyRG                = 11
HV_cmHIVPosRG               = 12
HV_cmModelFitting           = 13
HV_cmHIV                    = 14
HV_cmIA                     = 15
HV_cmCumIA                  = 16
HV_cmDA                     = 17
HV_cmCumDA                  = 18
HV_cmTotalCost              = 19
HV_cmCumCostIA              = 20
HV_cmCumCostDA              = 21
HV_cmTotalCostByInt         = 22
HV_cmPopAR                  = 23
HV_cmNewInfectionsAR        = 24
HV_cmPrevAR                 = 25
HV_cmHIVPosAR               = 26
HV_cmDistCD4NoART           = 27
HV_cmDistCD4ART             = 28
HV_cmAIDSDeathsAR           = 29
HV_cmSummary                = 30
HV_cmNumVaccsAR             = 31
HV_cmNumVaccsRG             = 32
HV_cmNumART                 = 33
HV_cmTotalCosts2            = 34
HV_cmAIDSDeathsART          = 35
HV_cmARTCoveragebyRG        = 36
HV_cmTotalARTCoverageAR     = 37
HV_cmARTCoverageCD4         = 38
HV_cmNewlyOnARTCD4          = 39
HV_cmNewlyOnARTTotal        = 40
HV_cmNewlyEligibleARTCD4    = 41
HV_cmNewlyEligibleARTTotal  = 42
HV_cmIncidenceRG            = 43
HV_cmIncidenceAR            = 44
# GVA = Goals vs AIM.  These indicators are only visible in developer's mode #
HV_cmGVAIncidence           = 45
HV_cmGVAPrevalence          = 46
HV_cmGVAHIVPopulation       = 47
HV_cmGVATotalPopulation     = 48
HV_cmGVANewInfections       = 49
HV_cmGVANumReceivingART     = 50
HV_cmGVAAIDSDeaths          = 51
# Multiplier is only visible in developer's mode #
HV_cmMultiplier             = 52
HV_cmMultiplierbyRG         = 53
HV_cmMultiplierAR           = 54
# for PrEP #
HV_cmTotalPrEP              = 55
HV_cmPrEPSingleMethod       = 56
HV_cmPrEPAllMethods         = 57
HV_cmPrEPRG                 = 58
HV_cmPrEPAR                 = 59
# Editor input indicators for Extract #
HV_exConfiguration          = 60
HV_exEpidemiology           = 61
HV_exBehavior               = 62
HV_exImpactMatrix           = 63

HV_MAXIND                   = 63

HV_OneMSMRiskGroup   = 0
HV_FourMSMRiskGroups = 1

# Condom use constants for logistics curve #
 # Logistics curve parameters #
HV_CU_A = 1         #Initial value
HV_CU_K = 2         #Final value
HV_CU_B = 3         #Growth rate
HV_CU_V = 4         #Growth location
HV_CU_M = 5         #Years to final value
HV_CU_Q = 6

# These female risk groups are used when AllRisk and MSM are needed for females.
#    MSM is usually used to round out the loop #
#  HV_AllRisk_F3      = 11
#  HV_None_F3         = 12
#  HV_LRH_F3          = 13
#  HV_MRH_F3          = 14
#  HV_HRH_F3          = 15
#  HV_IDU_F3          = 16
#  HV_MSM_F3          = 17

# These female risk groups are used when AllRisk and MSM are not needed for females
#   and HV_MSMIDU_M IS NOT part of the male risk groups #
#  HV_None_F1         = 10
#  HV_LRH_F1          = 11
#  HV_MRH_F1          = 12
#  HV_HRH_F1          = 13
#  HV_IDU_F1          = 14

# These female risk groups are used when AllRisk and MSM are not needed for females
#   and HV_MSMIDU_M IS part of the male risk groups #
#  HV_None_F2          = 11
#  HV_LRH_F2           = 12
#  HV_MRH_F2           = 13
#  HV_HRH_F2           = 14
#  HV_IDU_F2           = 15

# These risk groups are used when there is no differentiation between males and females. They
#   are also used where there are only males or part of the loop dedicated to males. #
HV_AllRisk         = 0
HV_None            = 1
HV_LRH             = 2
HV_MRH             = 3
HV_HRH             = 4
HV_IDU             = 5
HV_MSM             = 6
HV_MSMLR           = 7
HV_MSMMR           = 8
HV_MSMHR           = 9
HV_MSMIDU          = 10
HV_Total           = 11   #NOTE: HV_Total IS USED IN CALCULATIONS FOR MALE/FEMALE OFFSET
#  HV_Adults          = 12
#  HV_AllRisk_F3      = 11
HV_None_F3         = 12
HV_LRH_F3          = 13
HV_MRH_F3          = 14
HV_HRH_F3          = 15
HV_IDU_F3          = 16
HV_MSM_F3          = 17

HV_PercPop           = 1
HV_AvgDur            = 2

#Force Of Infection Sheet#
#  HV_Risk_I            = 1
#  HV_Male_I            = 2
#  HV_LRH_M_I           = 3
#  HV_MRH_M_I           = 4
#  HV_HRH_M_I           = 5
#  HV_IDU_M_I           = 6
#  HV_MSM_M_I           = 7
#  HV_Female_I          = 8
#  HV_LRH_F_I           = 9
#  HV_MRH_F_I           = 10
#  HV_HRH_F_I           = 11
#  HV_IDU_F_I           = 12

# Relative Infectiousness Sheet#
HV_PrimaryInf        = 1
HV_Asymptomatic      = 2
HV_SympNoART         = 3
HV_SympART           = 4

HV_Susceptibility    = 1
HV_Infect            = 2

HV_Neg               = 0
HV_Pos               = 1

# Prevalence by risk group Sheet#
HV_Males_P            = 1
HV_NotSexAct_M_P      = 2
HV_LRH_M_P            = 3
HV_MRH_M_P            = 4
HV_HRH_M_P            = 5
HV_IDU_M_P            = 6
HV_MSM_M_P            = 7
HV_MSMLR_M_P          = 8
HV_MSMMR_M_P          = 9
HV_MSMHR_M_P          = 10
HV_MSMIDU_M_P         = 11
HV_Total_M_P          = 12
HV_Females_P          = 13
HV_NotSexAct_F_P      = 14
HV_LRH_F_P            = 15
HV_MRH_F_P            = 16
HV_HRH_F_P            = 17
HV_IDU_F_P            = 18
HV_Total_F_P          = 19
HV_Adults_P           = 20

# MSM constants #
HV_OneMSM_P           = 1
HV_ThreeMSM_P         = 3
HV_FourMSM_P          = 4

#HIV Status#
HV_Negative           = 0
HV_Primary            = 1
HV_CD4_GT500          = 2
HV_CD4_350_500        = 3
HV_CD4_250_349        = 4
HV_CD4_200_249        = 5
HV_CD4_100_199        = 6
HV_CD4_50_99          = 7
HV_CD4_LT50           = 8
HV_CD4_GT500_ART      = 12
HV_CD4_350_500_ART    = 13
HV_CD4_250_349_ART    = 14
HV_CD4_200_249_ART    = 15
HV_CD4_100_199_ART    = 16
HV_CD4_50_99_ART      = 17
HV_CD4_LT50_ART       = 18
HV_AllHIV             = 19


# ART Selection constant #
#  HV_NumOrPercent       = 0
#  HV_CD4Category        = 1

#Progression Periods#
HV_YearsInPrimary       = 1

#Vaccine Start Year#
HV_VaccineStartYear     = 2015

#Initial Pulse#
HV_InitialPulse         = 0.01

# Risk Group Constants #
HV_RGReducCondomNonUse_High         = 1
HV_RGReducCondomNonUse_Med          = 2
HV_RGReducCondomNonUse_Low          = 3
HV_RGReducCondomNonUse_MSM          = 4
HV_RGReducCondomNonUse_IDU          = 5
HV_RGReducCondomNonUse_MSM_High     = 6
HV_RGReducCondomNonUse_MSM_Med      = 7
HV_RGReducCondomNonUse_MSM_Low      = 8
HV_RGReducCondomNonUse_MSM_IDU      = 9
HV_RGReducInNumPartners_High        = 10
HV_RGReducInNumPartners_Med         = 11
HV_RGReducInNumPartners_Low         = 12
HV_RGReducInNumPartners_MSM         = 13
HV_RGReducInNumPartners_IDU         = 14
HV_RGReducInNumPartners_MSM_High    = 15
HV_RGReducInNumPartners_MSM_Med     = 16
HV_RGReducInNumPartners_MSM_Low     = 17
HV_RGReducInNumPartners_MSM_IDU     = 18
HV_RGIncreaseAgeFirstSex            = 19
HV_RGReducUnsafeInjectBehavior      = 20
HV_RGReducNeedleSharing_IDU         = 21
HV_RGReducNeedleSharing_MSM_IDU     = 22
HV_RGReducNumSharingPartners        = 23
HV_MaxRiskGroups                    = 23

# Cost Effectiveness Analysis constants #
HV_MaxTargetYear  = 2500

# Sheets in HVModData file #
HV_EasyGoalsData              = 'EasyGoalsData'

# Help constants #
HV_Help_Configuration         = 'Configuration_for_Goals'
HV_Help_Epidemiology          = 'Epidemiology'
HV_Help_HIVPrevalence         = 'HIV_prevalence'
HV_Help_STIPrevalence         = 'STI_prevalence'
HV_Help_BloodTransfusion      = 'Blood_transfusion'
HV_Help_Behavior              = 'Behavior'
HV_Help_InjectingDrugUser     = 'Injecting_drug_user'
HV_Help_CondomUse             = 'Condom_use'
HV_Help_NumberOfPartners      = 'Number_of_partners'
HV_Help_SexActsPerPartner     = 'Sex_acts_per_partner'
HV_Help_AgeAtFirstSex         = 'Age_at_first_sex'
HV_Help_IncreasedRecruitment  = 'Increased_recruitment'
HV_Help_PercentMarried        = 'Percent_married'
HV_Help_Coverage              = 'Coverage'
HV_Help_ImpactMatrix          = 'Impact_matrix'
HV_Help_MaleCircumcision      = 'Male_circumcision'
HV_Help_PrEP                  = 'PrEP'
HV_Help_ART                   = 'ART'
HV_Help_Vaccines              = 'Vaccines'
HV_Help_FittingTheEpidemic    = 'Fitting_the_epidemic'

HV_DistanceFromEditor = 10

#**** Constants for Goals Express data generator ****#
HV_GE_DataFileTitle = 'Goals Express data generator file'

HV_GE_Version1      = 1.0
HV_GE_FileNumSpace  = 10
HV_GE_FileRDec      = 4

HV_GE_DefaultNumInterations = '500'

HV_GE_EXT_WILDCARD = '*.GE'
HV_GE_EXT          = 'GE'
HV_GE_CSV_EXT      = '_GE1.csv'

HV_GE_Delimeter   = '-1'

HV_GE_FinalYearOutput = 2050
HV_GE_YearIncrement   = 5

# Coverages used by the GE tool #
HV_GE_ART_M             = 1
HV_GE_ART_F             = 2
HV_GE_MC15_49           = 3
HV_GE_Condoms           = 4
HV_GE_MassMedia         = 5
HV_GE_YouthInSchool     = 6
HV_GE_SexWorkerOutreach = 7
HV_GE_MSMOutreach       = 8
HV_GE_IDUOutreach       = 9
HV_GE_MaxCoverages      = 9

# Output constants #
HV_GE_NewInfections_Out     = 1
HV_GE_AIDSDeaths_Out        = 2
HV_GE_NumReceivingART_Out   = 3
HV_GE_Males15_49_Out        = 4
HV_GE_Males15_Out           = 5
HV_GE_Condoms_Out           = 6
HV_GE_MassMedia_Out         = 7
HV_GE_YouthInSchool_Out     = 8
HV_GE_SexWorkerOutreach_Out = 9
HV_GE_MSMOutreach_Out       = 10
HV_GE_IDUOutreach_Out       = 11
HV_GE_MaxOutputs            = 11

# HVEdFM glow button constants #
HV_ConfigBtn        = 1
HV_EpidemiologyBtn  = 2
HV_BehaviorBtn      = 3
HV_CoverageBtn      = 4
HV_ImpactMatrixBtn  = 5
HV_UnitCostsBtn     = 6
HV_ResultsBtn       = 7
HV_FittingBtn       = 8

# Prior distribution indices, used for model fitting #
HV_PriorNormal = 0
HV_PriorGamma  = 1
HV_PriorBeta   = 2

# Fitting parameter indices #
HV_FitTransmitBase         =  0
HV_FitTransmitMtoF         =  1
HV_FitTransmitSTI          =  2
HV_FitTransmitMSM          =  3
HV_FitStageDurPrimary      =  4
HV_FitStageInfPrimary      =  5
HV_FitStageInfSymptomatic  =  6
HV_FitInitialPulse         =  7
HV_FitCoitalFreqLRH        =  8
HV_FitCoitalFreqMRH        =  9
HV_FitCoitalFreqHRH        = 10
HV_FitCoitalFreqMSM        = 11
HV_FitNeedleSharing        = 12
HV_FitSTIGrowthRate        = 13
HV_FitSTIGrowthLocation    = 14
HV_FitSTIYearstoFinalValue = 15
HV_MaxFitParams            = 16

# Size of the table used for fitting. TODO: make this dynamic #
HV_MaxFitData          = 31

# Constants for reading and writing EasyGoalsData #
HV_FileYearCol            = 0
HV_CountryCodeCol         = 0
HV_CountryNameCol         = 1
HV_EpidemiologyCol        = 2
HV_ARTEffectCol           = 15
HV_HIVPrevNoSexMaleCol    = 16
HV_HIVPrevLRHMaleCol      = 97
HV_HIVPrevMRHMaleCol      = 178
HV_HIVPrevHRHMaleCol      = 259
HV_HIVPrevIDUMaleCol      = 340
HV_HIVPrevMSMCol          = 421
HV_HIVPrevTotalMaleCol    = 502
HV_HIVPrevNoSexFemaleCol  = 583
HV_HIVPrevLRHFemaleCol    = 664
HV_HIVPrevMRHFemaleCol    = 745
HV_HIVPrevHRHFemaleCol    = 826
HV_HIVPrevIDUFemaleCol    = 907
HV_HIVPrevTotalFemaleCol  = 988
HV_HIVPrevAdultsCol       = 1069
HV_STIPrevalenceCol       = 1150
HV_BehaviorCol            = 1195
HV_AveDurBehaviorCol      = 1206
HV_IDUMalesCol            = 1213
HV_IDUFemalesCol          = 1294
HV_IDUShareCol            = 1375
HV_CondomUseCol           = 1456
HV_NumPartLRHMaleCol      = 1476
HV_NumPartMRHMaleCol      = 1557
HV_NumPartHRHMaleCol      = 1638
HV_NumPartMSMCol          = 1719
HV_NumPartLRHFemaleCol    = 1800
HV_NumPartMRHFemaleCol    = 1881
HV_NumPartHRHFemaleCol    = 1962
HV_SexActsLRHCol          = 2043
HV_SexActsMRHCol          = 2124
HV_SexActsHRHCol          = 2205
HV_SexActsLMSMCol         = 2286
HV_AgeFirstSexCol         = 2367
HV_IncreasedRecruitCol    = 2369
HV_PercentMarriedCol      = 2375
HV_MaxDatabaseCol         = 2384

# EasyGoalsData tags #
HV_CountryCodeTag         = '<Country Code>'
HV_CountryNameTag         = '<Country Name>'
HV_EpidemiologyTag        = '<Epidemiology>'
HV_ARTEffectTag           = '<ART effect for all t>'
HV_HIVPrevNoSexMaleTag    = '<HIV Prevalence-Not sexually active: Males>'
HV_HIVPrevLRHMaleTag      = '<HIV Prevalence-LRH: Males>'
HV_HIVPrevMRHMaleTag      = '<HIV Prevalence-MRH: Males>'
HV_HIVPrevHRHMaleTag      = '<HIV Prevalence-HRH: Males>'
HV_HIVPrevIDUMaleTag      = '<HIV Prevalence-IDU: Males>'
HV_HIVPrevMSMTag          = '<HIV Prevalence-MSM>'
HV_HIVPrevTotalMaleTag    = '<HIV Prevalence-Total: Males>'
HV_HIVPrevNoSexFemaleTag  = '<HIV Prevalence-Not sexually active: Females>'
HV_HIVPrevLRHFemaleTag    = '<HIV Prevalence-LRH: Females>'
HV_HIVPrevMRHFemaleTag    = '<HIV Prevalence-MRH: Females>'
HV_HIVPrevHRHFemaleTag    = '<HIV Prevalence-HRH: Females>'
HV_HIVPrevIDUFemaleTag    = '<HIV Prevalence-IDU: Females>'
HV_HIVPrevTotalFemaleTag  = '<HIV Prevalence-Total: Females>'
HV_HIVPrevAdultsTag       = '<HIV Prevalence-Adults>'
HV_STIPrevalenceTag       = '<STI Prevalence>'
HV_BehaviorTag            = '<Behavior>'
HV_AveDurBehaviorTag      = '<Average duration of behavior>'
HV_IDUMalesTag            = '<IDU-force of infection: Males>'
HV_IDUFemalesTag          = '<IDU-force of infection: Females>'
HV_IDUShareTag            = '<IDU-Sharing needles>'
HV_CondomUseTag           = '<Condom use>'
HV_NumPartLRHMaleTag      = '<Number of partners LRH Males>'
HV_NumPartMRHMaleTag      = '<Number of partners MRH Males>'
HV_NumPartHRHMaleTag      = '<Number of partners HRH Males>'
HV_NumPartMSMTag          = '<Number of partners MSM>'
HV_NumPartLRHFemaleTag    = '<Number of partners LRH Females>'
HV_NumPartMRHFemaleTag    = '<Number of partners MRH Females>'
HV_NumPartHRHFemaleTag    = '<Number of partners HRH Females>'
HV_SexActsLRHTag          = '<Sex acts per partner LRH>'
HV_SexActsMRHTag          = '<Sex acts per partner MRH>'
HV_SexActsHRHCTag         = '<Sex acts per partner HRH>'
HV_SexActsLMSMTag         = '<Sex acts per partner MSM>'
HV_AgeFirstSexTag         = '<Age ate first sex>'
HV_IncreasedRecruitTag    = '<Increased recruitment>'
HV_PercentMarriedTag      = '<Percent married>'



import numpy as np
from scipy import sparse

def make_model(p, r, i, s, gps):
    
    # --- Get the linear rates ------------------------------------------------
    size = [i['nstates']]*2
    m    = np.zeros(size)
    m2 = np.zeros(size)
    #m2pr = np.zeros(size)

    for istr in gps['strains']:#for ip in range(len(gps['strains'])):
        strain = istr#gps['strains'][ip]
        for ih in gps['hiv']:#for ih in range(len(gps['hiv'])):
            hiv = ih#gps['hiv'][ih]
            idx_ih = (ih=='h1') + 2*(ih=='hart') 
            Isc = i['Isc'][hiv][strain]#i['st']['hiv']['strain']['Isc']
            I = i['I'][hiv][strain]#i['st']['hiv']['strain']['I']
            E = i['E'][hiv][strain]
            Dxs = i['Dx'][hiv][strain]
            Txs = i['Tx'][hiv][strain]
            Tx2s = i['Tx2'][hiv][strain]
            Rlo = i['Rlo'][hiv][strain]
            Rhi = i['Rhi'][hiv][strain]
            R = i['R'][hiv][strain]

            #% Group Selectors
            ismdr = strain=='mdr'
            for iv in gps['vacc']:
                #vacc = gps['vacc'][iv]
                idx_iv = (iv=='v1') + 2*(iv=='v2') 
                Lf = i['Lf'][hiv][iv][strain]
                Ls = i['Ls'][hiv][iv][strain]

                #% --- Fast progression and LTBI stabilisation
                m[Isc][Lf] = m[Isc][Lf] + r['progression'][idx_ih]*p['PT_PLHIV'][idx_ih]*(1-(idx_iv==1)*p['VE'][1])
                m[Ls][Lf] = m[Ls][Lf] + r['LTBI_stabil'][idx_ih]
    
                # --- Reactivation
                m[Isc][Ls] = m[Isc][Ls] + r['reactivation'][idx_ih]*p['PT_PLHIV'][idx_ih]*(1-(idx_iv==1)*p['VE'][1])

            #% --- Sub-clinical to clinical Infection    
            m2[I][Isc] = m2[I][Isc] + r['r_sym']

            # % Introduce public/private sector
            # % --- Primary careseeking, including access to public sector care
            m2[Dxs['pu']][I] = m2[Dxs['pu']][I] + r['cs']*p['pu'] + r['access']
            m2[Dxs['pr']][I] = m2[Dxs['pr']][I] + r['cs']*(1-p['pu'])

            #% --- Secondary careseeking
            m2[Dxs['pu']][E] = m2[Dxs['pu']][E] + r['cs2']*p['pu']
            m2[Dxs['pr']][E] = m2[Dxs['pr']][E] + r['cs2']*(1-p['pu'])

            #% --- Case finding from subclinical TB
            m2[Dxs['pu']][Isc] = m2[Dxs['pu']][Isc] + r['cs3']*p['pu']
            m2[Dxs['pr']][Isc] = m2[Dxs['pr']][Isc] + r['cs3']*(1-p['pu'])

            for ip in gps['provs']:#for ip in range(len(gps['provs'])):
                prov = ip
                idx_ip = 0+(ip=='pr')
                Dx = Dxs[prov]
                Tx = Txs[prov]
                Tx2 = Tx2s[prov]

                try: 
                  pFLinit = p['Dx'][idx_ip]*p['Tx_init'][idx_ip]*(1-ismdr*p['MDR_rec'][idx_ip])
                except:
                    KeyboardInterrupt

                pSLinit = p['Dx'][idx_ip]*p['Tx_init2'][idx_ip]*ismdr*p['MDR_rec'][idx_ip]    #pSLinit = p.Dx(ip)*p.Tx_init2(ip)*ismdr*p.MDR_rec(ip);
                p_ltfu  = 1-(pFLinit+pSLinit) # %p.Dx(ip)*p.Tx_init(ip) #p_ltfu  = 1-(pFLinit+pSLinit); %p.Dx(ip)*p.Tx_init(ip);
                m[E][Dx] = m[E][Dx] + r['Dx']*p_ltfu
                m[Tx2][Dx] = m[Tx2][Dx] + r['Dx']*pSLinit
                m[Tx][Dx] = m[Tx][Dx] + r['Dx']*pFLinit
                #% --- FL Treatment
                pFLcure = p['cure'][idx_ip]*(1-ismdr) #            pFLcure = p.cure(ip)*(1-ismdr);
                pSLtran = p['SL_trans'][idx_ip]*ismdr #pSLtran = p.SL_trans(ip)*ismdr;
                rMDRacq = r['MDR_acqu']*(1-ismdr) #

                #source  = Tx;
                #destins = [Rlo            Tx2                        E                              Rhi,             i.Tx.(hiv).mdr.(prov)];
                #rates   = [r.Tx*pFLcure,  r.Tx*(1-pFLcure)*pSLtran,  r.Tx*(1-pFLcure)*(1-pSLtran),  r.default(ip),   rMDRacq];
                #m(destins, source) = m(destins, source) + rates';
                m[i['Tx'][hiv]['mdr'][prov]][Tx] = m[i['Tx'][hiv]['mdr'][prov]][Tx] + rMDRacq
                m[Rhi][Tx] = m[Rhi][Tx] + r['default'][idx_ip]
                m[E][Tx] = m[E][Tx] + r['Tx']*(1-pFLcure)*(1-pSLtran)
                m[Tx2][Tx] = m[Tx2][Tx] + r['Tx']*(1-pFLcure)*pSLtran
                m[Rlo][Tx] = m[Rlo][Tx] + r['Tx']*pFLcure

                #% --- SL Treatment
                #source  = Tx2;
                #destins = [Rlo                 E];          %Rhi];
                #rates   = [r.Tx2*p.cure2(ip),  r.Tx2*(1-p.cure2(ip))+ r.default2(ip)];
                #m(destins, source) = m(destins, source) + rates';
                m[E][Tx2] = m[E][Tx2] + r['Tx2']*(1-p['cure2'][idx_ip])+ r['default2'][idx_ip]
                m[Rlo][Tx2] = m[Rlo][Tx2] + r['Tx2']*p['cure2'][idx_ip]
                #End Provider Loop

            #% --- Relapse
            #sources = [Rlo, Rhi, R];
            #destin  = Isc;
            #rates   = r.relapse;
            #m(destin, sources) = m(destin, sources) + rates;
            #m[Isc][R] = m[Isc][R] + r['relapse']
            #m[Isc][Rhi] = m[Isc][Rhi] + r['relapse']
            #m[Isc][Rlo] = m[Isc][Rlo] + r['relapse']
             
            m[Isc][[Rlo,Rhi,R]] = m[Isc][[Rlo,Rhi,R]] + r['relapse']
            
            #sources = [Rlo, Rhi];
            #destin  = R;
            #rates   = 0.5;
            #m(destin, sources) = m(destin, sources) + rates;
            #m[R][Rhi] = m[R][Rhi] + 0.5
            #m[R][Rlo] = m[R][Rlo] + 0.5
            m[R][[Rhi,Rlo]] = m[R][[Rhi,Rlo]] + 0.5
        
            #% --- Self cure
            #sources = intersect(intersect(s.infectious,s.(hiv)), s.(strain));
            #destin  = Rhi;
            #rates   = r.self_cure(ih);
            #m(destin, sources) = m(destin, sources) + rates; 
            #for source in s['infectious'].intersection(s['hiv']).intersection(s['strain']): 
            #    m[Rhi][source] = m[Rhi][source] + r['self_cure'][ih]  
            source = np.intersect1d(np.intersect1d(s['infectious'],s[hiv]),s[strain])    
            m[Rhi][source] = m[Rhi][source] + r['self_cure'][idx_ih] 
        #End hiv loop
    #End MDR loop        

    for ih in gps['hiv']:#for ih in range(len(gps['hiv'])):
        hiv = ih#gps['hiv'][ih]
        source = i['U'][hiv]['v0']
        destin = i['U'][hiv]['v1']
        m[destin][source] = m[destin][source] + r['vacc']

        source = i['U'][hiv]['v1']
        destin = i['U'][hiv]['v2']
        m[destin][source] = m[destin][source] + r['waning']

        for istr in gps['strains']:#for istr in range(len(gps['strains'])):
            strain = istr#gps['strains'][istr]
            source = i['Lf'][hiv]['v0'][strain]
            destin = i['Lf'][hiv]['v1'][strain]
            m[destin][source] = m[destin][source] + r['vacc']

            source = i['Lf'][hiv]['v1'][strain]
            destin = i['Lf'][hiv]['v2'][strain]
            m[destin][source] = m[destin][source] + r['waning']

    #% --- HIV acquisition                                                      % <--- NEW
    m3 = np.zeros((i['nstates'],i['nstates'])) #m3 = zeros(i.nstates);
    sources = s['h0'] #sources = s.h0;
    destins = s['h1'] #destins = s.h1;
    #inds    = np.sub2ind(size(m), destins, sources) #;#inds    = sub2ind(size(m), destins, sources);
    inds = np.ravel_multi_index([destins, sources], dims=np.shape(m3), order='F')
    #rates   = 1;
    #m3[inds] = m3[inds] + 1#m3(inds) = m3(inds) + rates;
    #m3.ravel()[inds] = m3.ravel()[inds] + 1#m3(inds) = m3(inds) + rates;
    m3[destins,sources] = m3[destins,sources] + 1

    #% --- ART initiation
    sources = s['h1']#sources = s.h1;
    destins = s['hart']#destins = s.hart;
    #inds    = np.sub2ind(size(m), destins, sources) #inds    = sub2ind(size(m), destins, sources);
    inds = np.ravel_multi_index([destins, sources], dims=np.shape(m), order='F')
    #rates   = r.ART_init;
    #m[inds] = m[inds] + r['ART_init']; #m(inds) = m(inds) + rates;
    #m.ravel()[inds] = m.ravel()[inds] + r['ART_init']; #m(inds) = m(inds) + rates;
    m[destins, sources] = m[destins, sources] + r['ART_init']
    

    #% --- Bring them together
    M = {}
    M['lin']   = sparse.csc_matrix(m - np.diag(np.sum(m,0))) #M.lin    = sparse(m - diag(sum(m,1)));
    M['Dxlin'] = sparse.csc_matrix(m2 - np.diag(np.sum(m2,0))) #M.Dxlin  = sparse(m2 - diag(sum(m2,1)));
    M['linHIV'] = sparse.csc_matrix(m3 - np.diag(np.sum(m3,0))) #M.linHIV = sparse(m3 - diag(sum(m3,1)));                                   % <--- NEW, now HIV acquisition is separate matrix made in lines 62-67

    M['nlin'] = {}
    #% --- Get the nonlinear rates ---------------------------------------------
    #% --- Allocating transitions
    for istr in gps['strains']:#for istr in range(len(gps['hiv'])):
        strain = istr

        m = np.zeros((i['nstates'],i['nstates']))
        for ih in gps['hiv']:#for istr in range(len(gps['hiv'])):
            hiv = ih
            idx_ih = (ih=='h1') + 2*(ih=='hart')
            for iv in gps['vacc']:#for iv in range(len(gps['vacc'])):
               vacc = iv
               idx_iv = (iv=='v1')+2*(iv=='v2')
               #sources = [i['U'][hiv][vacc], (([s['Lf'], s['Ls'], s['Rlo'], s['Rhi'], s['R']].intersecion(s[hiv])).intersection(s[strain])).intersection(s[vacc])]
               sources = np.concatenate([[i['U'][hiv][vacc]], np.intersect1d(np.intersect1d(np.intersect1d(np.concatenate([s['Lf'], s['Ls'], s['Rlo'], s['Rhi'], s['R']]),s[hiv]),s[strain]),s[vacc])])
               destin = i['Lf'][hiv][vacc][strain] #destin  = i.Lf.(hiv).(vacc).(strain);
               m[destin][sources] = m[destin][sources] + (1-(idx_iv==1)*p['VE'][0])
               #%M.nlin.(hiv).(strain) = sparse(m - diag(sum(m,1)));
        

            #% Adjust for any immune protection
            cols = np.intersect1d(np.concatenate([s['Lf'], s['Ls'], s['Rlo'], s['Rhi'], s['R']]),s[hiv])
            m[:,cols] = m[:,cols]*(1-p['imm'][idx_ih])#m(:,cols) = m(:,cols)*(1-p.imm(ih));

        M['nlin'][strain] = sparse.csc_matrix(m - np.diag(np.sum(m,0)))#M.nlin.(strain) = sparse(m - diag(sum(m,1)));

    #% --- Getting force-of-infection
    m = np.zeros((2,i['nstates'])) #m = zeros(2,i.nstates);
    m[0,np.intersect1d(s['infectious'],s['ds'])]  = r['beta'] #m(1,intersect(s.infectious, s.ds))  = r.beta;
    m[1,np.intersect1d(s['infectious'],s['mdr'])] = r['beta']*p['rfbeta_mdr'] #m(2,intersect(s.infectious, s.mdr)) = r.beta*p.rfbeta_mdr;
    m[:,[s['h1'], s['hart']]] = m[:,[s['h1'], s['hart']]]*p['rfbeta_hiv'] #m(:,[s.h1, s.hart]) = m(:,[s.h1, s.hart])*p.rfbeta_hiv;
    m[:,s['Isc']] = m[:,s['Isc']]*0.75 #m(:,s.Isc) = m(:,s.Isc)*0.75;
    
    M['lambda'] = sparse.csc_matrix(m)#M.lambda = sparse(m);
    
    #% --- Get the mortality rates
    m = np.zeros((i['nstates'],3))
    m[:,0]            = r['mort']#m(:,1)            = r.mort;
    m[s['h1'],0]      = r['HIV_mort']#;                                            % HIV+ve, no TB
    #% HIV-ve TB
    inds = np.intersect1d(s['h0'],s['infectious_wosc'])#inds = intersect([s.h0], s.infectious_wosc);
    m[inds,1] = r['mort_TB'][0]#m(inds,2) = r.mort_TB(1);
    #% HIV/TB coinfection
    inds = np.intersect1d(np.concatenate([s['h1'],s['hart']]),s['infectious_wosc'])
    m[inds,2] = r['mort_TB'][1]#m(inds,3) = r.mort_TB(2);
    #%keyboard
    M['mortvec'] = m#M.mortvec = m;

    return M



from datetime import date
from copy import deepcopy

import SpectrumCommon.Const.GB as gbc
import SpectrumCommon.Const.PJ.PJNTags as pjmvt
import SpectrumCommon.Modvars.GB.GBUtil as gbu

import SpectrumCommon.Const.IH.IHConstLink as ihcl
import SpectrumCommon.Const.IH.IHTags as ihmvt
import SpectrumCommon.Modvars.IH.IHModVarDefs as ihmvd
import SpectrumCommon.Modvars.IH.IHModVarUtil as ihmvu

import SpectrumCommon.Modvars.IH.IHReadDatabases as ihrd

def init_modvars(countryID, first_year, final_year, extra):
    '''
        Initializes IHT Configuration MVs for new IHT projections.

            Parameters:
                countryID (string):  Ex: BEN for Benin.

                first_year (int): First year selected by user.

                final_year (int): Final year selected by user. 

                extra (dict): Any other fields.  
 
            Returns:
                mod_vars_rec (dict): MVs.
    '''
    
    # Load databases

    IHT_costing_on = extra[gbc.GB_IHT_ON] if gbc.GB_IHT_ON in extra else False
    TB_costing_on = extra[gbc.GB_TB_COSTING_ON] if gbc.GB_TB_COSTING_ON in extra else False
    LiST_costing_on = extra[gbc.GB_LIST_COSTING_ON] if gbc.GB_LIST_COSTING_ON in extra else False
    costing_mode_mod_vars = {
        pjmvt.GB_IHTOnTag       : IHT_costing_on,
        pjmvt.GB_TBCostingOnTag : TB_costing_on,
        pjmvt.GB_LCOnTag        : LiST_costing_on,
    }

    costing_mode = gbu.get_costing_mode(costing_mode_mod_vars)

    group_DB, group_DB_IHT_ID_info_only = ihrd.IH_load_group_DB(costing_mode)

    value_bool = False
    value_int = 0
    value_flt = 0.0
    value_str = ''
    value_date = date.today().strftime("%B %d, %Y"),   

    prog_area_rec_list = ihrd.IH_init_prog_area_records(group_DB)
    deliv_chan_rec_list = ihmvd.IH_init_deliv_chan_records(costing_mode)
    health_sys_admin_unit_rec_list = ihmvd.IH_init_health_sys_admin_unit_records()

    # If TB costing is on, we'll need access to the IHT programme areas well, because the user is allowed to move IHT interventions into TB. 
    prog_area_records_IHT_ID_info_only = []

    if TB_costing_on:
        prog_area_records_IHT_ID_info_only = ihrd.IH_init_prog_area_records_ID_info_only(group_DB_IHT_ID_info_only)

    # deliv_chan_pack_record_list []
    # nat_prog_record_list = []
    # audit_record_list = []
    # locked_area_record_list = []

    num_prog_areas = ihmvu.get_IH_num_prog_areas(prog_area_rec_list)
    #num_years_inputs = final_year - first_year + 1 
    num_years_outputs = final_year - first_year + 1

    staff_cap_utiliz_list = ihmvd.IH_init_prog_area_staff_year_arr(num_prog_areas, ihcl.HW_NUM_STATIC_STAFF_TYPES, num_years_outputs, 0.0)

    # add flag modvars to one modes modvar. If flag is in set, turn mode on. Else, turn mode off. Just like how you will do with intervention sets in IC.

    mod_vars_rec = {
        ihmvt.IH_VERSION_TAG_V1                                    : 1,
        ihmvt.IH_HLTH_SERV_ANAL_MODE_MST_ID_TAG_V1                 : value_int,  # Programme or Delivery Channel Mode
        ihmvt.IH_PROG_AREA_DELIV_CHAN_FILTER_MST_ID_TAG_V1         : value_int, # programme area or delivery channel mstID
        ihmvt.IH_MAX_CUST_INTERV_MST_ID_TAG_V1                     : value_int,      
        ihmvt.IH_MOH_BUDGET_TAG_V1                                 : value_flt,                 
        ihmvt.IH_DATA_CHANGED_TAG_V1                               : value_bool, 
        ihmvt.IH_PROJ_VALID_TAG_V1                                 : value_bool, 
        ihmvt.IH_USING_ADOL_STRATA_MODE_TAG_V1                     : value_bool, 
        ihmvt.IH_USING_DIS_SPEC_COST_MODE_TAG_V1                   : value_bool, 
        ihmvt.IH_USING_PHC_MODE_TAG_V1                             : value_bool, 
        ihmvt.IH_USING_DEF_MED_STAFF_TYPES_TAG_V1                  : value_bool, 
        ihmvt.IH_USING_DEF_FAC_TYPES_TAG_V1                        : value_bool, 
        ihmvt.IH_USING_MOH_BUDGET_RES_TAG_V1                       : value_bool, 
        ihmvt.IH_USING_HPV_PROG_AREA_TAG_V1                        : value_bool, 
        ihmvt.IH_USING_WORK_HLTH_PROG_AREA_TAG_V1                  : value_bool, 
        ihmvt.IH_USING_TP_FROM_TIME_TAG_V1                         : value_bool, 
        ihmvt.IH_USING_TB_IMPACT_CALCS_TP_TAG_V1                   : value_bool, 
        ihmvt.IH_USING_TB_DAT_TP_TAG_V1                            : value_bool, 
        ihmvt.IH_USING_LOCAL_CURR_TAG_V1                           : value_bool, 
        ihmvt.IH_FILE_NAME_TAG_V1                                  : value_str,                          
        ihmvt.IH_PROJ_VALID_DATE_TAG_V1                            : value_date,             
        ihmvt.IH_PROG_AREA_RECORDS_TAG_V1                          : prog_area_rec_list,              
        ihmvt.IH_DELIV_CHAN_RECORDS_TAG_V1                         : deliv_chan_rec_list,            
        ihmvt.IH_PERC_PROG_MGMT_COSTS_PHC_VIS_TAG_V1               : value_bool,       
        ihmvt.IH_HR_FACTOR_PHC_TAG_V1                              : value_flt,         
        ihmvt.IH_LG_PERC_DRUG_SUPP_COSTS_PHC_TAG_V1                : value_int,    
        ihmvt.IH_SHOW_INACTIVE_PROG_AREAS_EXIST_INTERV_V1          : value_bool,
        ihmvt.IH_SHOW_INACTIVE_PROG_AREAS_EXIST_DRUG_SUPP_V1       : value_bool,
        ihmvt.IH_ENTER_UNIT_COSTS_ALL_YEARS_DRUG_SUPP_V1           : value_bool,
        ihmvt.IH_HEALTH_SYS_ADMIN_UNIT_RECORDS_TAG                 : health_sys_admin_unit_rec_list,

        # Results
        ihmvt.IH_RES_STAFF_CAP_UTILIZATION_STATIC_TAG_V1           : deepcopy(staff_cap_utiliz_list),
        ihmvt.IH_RES_STAFF_CAP_UTILIZATION_DYNAMIC_TAG_V1          : deepcopy(staff_cap_utiliz_list),
    }

    if TB_costing_on:
        # Disease-specific costing (DSC), TB Costing

        # TB costing
        
        # If TB costing is on, we'll need access to the IHT programme areas as well, because the user is allowed to move IHT interventions into TB. 
        mod_vars_rec[ihmvt.IH_PROG_AREA_RECORDS_IHT_ID_INFO_ONLY_TAG_V1] = prog_area_records_IHT_ID_info_only

    #Debugging
    #with open('C:\Proj\IHTSampleFile.JSON', 'w') as f:
    #   json.dump(mod_vars_dict, f)

    return mod_vars_rec

    



import numpy as np

from SpectrumCommon.Const.TB import *

def TB_PreventionDefaults(rp_Rif, year_len):
    preventionTargets =  np.zeros((TB_ResistanceLen, TB_ResistAgeLen, year_len))
    preventionTargets[TB_SEN][TB_Child] = rp_Rif[TB_SEN][TB_ResistAge0_14] 
    preventionTargets[TB_SEN][TB_Adult] = rp_Rif[TB_SEN][TB_ResistAge15p] 

    preventionTargets[TB_MDR][TB_Child] = rp_Rif[TB_MDR][TB_ResistAge0_14] 
    preventionTargets[TB_MDR][TB_Adult] = rp_Rif[TB_MDR][TB_ResistAge15p]

    # preventionTargets[TB_SEN][TB_Child] = 1 - preventionTargets[TB_MDR][TB_Child]
    # preventionTargets[TB_SEN][TB_Adult] = 1 - preventionTargets[TB_MDR][TB_Adult]
 
    return preventionTargets


PJN_TitleTag = "<PJN_Title_V1>"
PJN_AppIDTag = "<PJN_AppID_V1>"
PJN_ProjectionValidTag = "<PJN_ProjectionValid_V1>"
PJN_ProjectionSavedTag = "<PJN_ProjectionSaved_V1>"
PJN_AIMImpactCalculatedTag = '<PJN_AIMImpactCalculated_V1>'

PJN_UserSourceTag = "<PJN_UserSource_V1>"

# Versions'<PJN_ _V1>'
PJN_VersionTag = "<PJN_Version_V1>"
PJN_VersionNumberTag = "<PJN_VersionNumber_V1>"
PJN_BetaVersionTag = "<PJN_BetaVersion_V1>"

# years'<PJN_ _V1>'
# PJN_BaseYearTag = "<PJN_BaseYear_V1>"
PJN_FirstYearTag = "<PJN_FirstYear_V1>"
PJN_FinalYearTag = "<PJN_FinalYear_V1>"

# country info'<PJN_ _V1>'
PJN_CountryNameTag = "<PJN_CountryName_V1>"
PJN_ISO3NumericTag = "<PJN_ISO3Numeric_V1>"
PJN_ISO3AlphaTag = "<PJN_ISO3Alpha_V1>"


PJN_IHTPlanTypeTag = "<PJN_IHTPlanType_V1>"

PJN_SubNationalCodeTag = "<PJN_SubNationalCode_V1>"
PJN_SubNationalNameTag = "<PJN_SubNationalName_V1>"

PJN_ModulesTag = "<PJN_Modules_V1>"

PJN_TBSubnatTag = "<PJN_TBSubnat_V1>"

PJN_AuthorTag = "<PJN_Author_V1>"
PJN_OrganizationTag = "<PJN_Organization_V1>"
PJN_NotesTag = "<PJN_Notes_V1>"
PJN_CKAN_NameID_Tag = "<PJN_CKAN_NameID_V1>"
PJN_LibraryIDTag = "<PJN_LibraryID_V1>"
PJN_LibraryVersionIDTag = "<PJN_LibraryVersionID_V1>"

PJN_ReadOnlyTag = "<PJN_ReadOnly_V1>"
PJN_CKANCapacityTag = "<PJN_CKAN_CAPACITY_V1>"
PJN_Modes_Tag = "<PJN_Modes_V1>"

GB_UseLeapfrogTag = "<GB_UseLeapfrog_V1>"
GB_DataChangedTag = "<GB_DataChanged_V1>"
GB_CalcStateValidTag = "<GB_CalcStateValid_V1>"
GB_CustomEasyDataSetTag = "<GB_CustomEasyDataSet_V1>"

GB_CurrencyNameTag = "<GB_CurrencyName_V1>"
GB_DiscountRateCostsTag = "<GB_DiscountRateCosts_V1>"
GB_ExchangeRateTag = "<GB_ExchangeRate_V1>"
GB_InflationTag = "<GB_Inflation_V1>"
GB_InflationMethodTag = "<GB_InflationMethod_V1>"

GB_DisabilityWeightsTag = "<GB_DisabilityWeights_V1>"
GB_DisabilityWeightsSetTag = "<GB_DisabilityWeightsSet_V1>"

GB_LCOnTag = "<GB_LCOn_V1>"
GB_GSKModeTag = '<GB_GSKMode_V1>'
GB_TBCostingOnTag = "<GB_TBCostingOn_V1>"
GB_IHTOnTag = "<GB_IHTOn_V1>"
# GB_RAPIDSectorsOnTag = "<GB_RAPIDSectorsOn_V1>"

GB_SubnatWizProjTag = "<GB_SubnatWizProj_V1>"
GB_SubnatRegionTag = "<GB_SubnatRegion_V1>"
GB_SubnatRegionCodeTag = "<GB_SubnatRegionCode_V1>"
GB_SubnatSurveydTag = "<GB_SubnatSurveyd_V1>"

GB_TBImpactStatisticalTag = "<GB_TBImpactStatistical_V1>"
GB_TBImpactDynamicalTag   = "<GB_TBImpactDynamical_V1>"
GB_UserSourcesTag         = "<GB_UserSources_V1>"
GB_TaskGUIDsTag           = "<GB_TaskGUIDs_V1>"




# Used for CustomEasyData imported from projection #
RN_ImportedProj  = 1

# Constants for writing to file #
RN_Version       = 1.0
RN_DataFileTitle = 'Resource Needs Model'

GB_FirstProj = 1

# Time constants #
RN_Min_Year         = 1

# Region constant #
RN_Total            = 0

# Sex constants #
RN_BothSexes        = 0
RN_Male             = 1
RN_Female           = 2

# Display constants #

# Sectors #
RN_GeneralSect  = 1
RN_ResourcesRequiredSect       = 1
RN_NumberPeopleReachedSect     = 2
RN_AdultTreatmentCostsSect     = 3
RN_ChildPMTCTandTreatCostsSect = 4
RN_NUM_SECT                    = 4
RN_MAXSEC                      = 4

# Indicators #
RN_FirstInd                       = 1
RN_SummaryResourcesRequiredInd    = 1
RN_SummaryNumberPeopleReachedInd  = 2
# Adult Treatment Costs #
RN_FirstLineARVInd                = 3
RN_SecondLineARVInd               = 4
RN_TBArvInd                       = 5
RN_LabTestInd                     = 6
RN_OItreatmentInd                 = 7
RN_CotrimoxazoleInd               = 8
RN_TBProphylaxisInd               = 9
RN_NutritionInd                   = 10
RN_FirstlineDeliveryCostsInd      = 11
RN_SecondlineDeliveryCostsInd     = 12
RN_OITreatmentDeliveryCostsInd    = 13
RN_TotalAdultTreatmentCostsInd    = 14
RN_SummaryCostsInd                = 15

# Child PMTCT and Treatment Costs #
RN_PMTCTCostsInd                  = 16
RN_ChildTreatmentCostsInd         = 17
RN_TotalChildTreatmentCostsInd    = 18
RN_PMTCTBenefitCostRatioInd       = 19
RN_SummaryPMTCTTableInd           = 20
RN_NUM_IND                        = 20

# Indicator constants #
RN_SEXIND       = 1
RN_SUMIND       = 2

RN_Indent1  = '    '          # 4 spaces
RN_Indent2  = '        '      # 8 spaces
RN_Indent3  = '            '  #12 spaces

# Graphical constants #
RN_ofTable           = 101
RN_OfPyramidPer      = 102
RN_OfPyramidNum      = 103
RN_OfSummaryTable    = 104
RN_OfStacked2D       = 105
RN_OfStacked3D       = 106

# Editor constants for getting and setting source data. #
RN_FirstEd     = 1
RN_NUM_EDITORS = 1

RN_USD           = 'USD'
RN_USCurrency    = 0
RN_LocalCurrency = 1

RN_DefaultProgramSupport   = 0
RN_UserAddedProgramSupport = 1

RN_BaseYear = 2015  #this will be updated to reflect current values in the database once they are avaialble

# Program support constants #
RN_PS_EnablingEnvironment             = 1
RN_PS_ProgramManagement               = 2
RN_PS_Research                        = 3
RN_PS_MonitoringAndEvaluation         = 4
RN_PS_StrategicCommunication          = 5
RN_PS_Logistics                       = 6
RN_PS_ProgramLevelHR                  = 7
RN_PS_Training                        = 8
RN_PS_LaboratoryEquipment             = 9
RN_PS_MaxProgramSupport               = 9
RN_PS_FirstRowUserAddedProgramSupport = 10

# Population sizes constants #
# General populations #
RN_POP_SexuallyActive15_49            = 1
RN_POP_Males15_49RegPartners          = 2
RN_POP_Males15_49NonRegPartners       = 3
RN_POP_NumSexActsRegPartners          = 4
RN_POP_NumSexActsNonRegPartners       = 5
RN_POP_CommercialSexActsFSW           = 6
RN_POP_CommercialSexActsMSW           = 7
RN_POP_CondomWastage                  = 8
RN_POP_PrimarySchoolMale              = 9
RN_POP_PrimarySchoolFemale            = 10
RN_POP_PrimaryPupilTeacher            = 11
RN_POP_SecondarySchoolMale            = 12
RN_POP_SecondarySchoolFemale          = 13
RN_POP_SecondaryPupilTeacher          = 14
RN_POP_TeacherRetraining              = 15
RN_POP_FormalSectorEmployees          = 16
RN_POP_Rapes                          = 17
# Most At-Risk populations #
RN_POP_FemaleSexWorkers               = 18
RN_POP_MaleSexWorkers                 = 19
RN_POP_MSM                            = 20
RN_POP_IDU                            = 21
RN_POP_IDUOpiodDependent              = 22
RN_POP_NumInjectionsPerYear           = 23
# Male circumcision #
RN_POP_AdultMalesCircum               = 24
# Medical Services #
RN_POP_NewCasesSTIsMale               = 25
RN_POP_NewCasesSTIsFemale             = 26
RN_POP_STIsSymptomaticMale            = 27
RN_POP_STIsSymptomaticFemale          = 28
RN_POP_BloodUnitsRequired             = 29
RN_POP_PEPKitsRequired                = 30
RN_POP_NumImmunChild0_23Mths          = 31
RN_POP_NumAdultInjections             = 32
RN_POP_InjectionsUnsafe               = 33
RN_POP_HospitalBeds                   = 34
# Max populations constants #
RN_POP_MaxPopulations                 = 34
RN_POP_MaxGeneralPop                  = 17
RN_POP_MaxRowGeneralPop               = 21
RN_POP_FirstRowUserAddedGeneralPop    = 22
RN_POP_MaxMostAtRiskPop               = 6
RN_POP_MaxRowMostAtRiskPop            = 6
RN_POP_FirstRowUserAddedMostAtRiskPop = 7
RN_POP_MaxRowNumPreSetKeyPops         = 7
RN_POP_FirstRowNumAddedKeyPops        = 8

RN_UserGeneralPop     = 1
RN_UserMostAtRiskPop  = 2

RN_Number  = 0
RN_Percent = 1

# Intervention constants #
# General Populations #
RN_ComMob                = 1
RN_MassMedia             = 2
RN_VCT                   = 3
RN_Condom                = 4
RN_PrimaryTeachers       = 5
RN_SecondaryTeachers     = 6
RN_OutOfSchoolYouth      = 7
RN_EconomicStrengthening = 8
RN_WorkplaceSTI          = 9
# Most At-Risk populations #
RN_FSW              = 10
RN_MSW              = 11
RN_MSMOutreach      = 12
RN_MSMLub           = 13
RN_IDUHarmRed       = 14
RN_IDUCandT         = 15
RN_IDUOutreach      = 16
RN_IDUNSEP          = 17
RN_IDUDrugSub       = 18
# Male circumcision #
RN_MC15_49          = 19
RN_MCInfant         = 20
# PrEP #
#  RN_PrEPOral         = 21
#  RN_PrEPInject       = 22
#  RN_PrEPGel          = 23
#  RN_PrEPRing         = 24
RN_PrEPOralDaily    = 21
RN_PrEPOralMonthly  = 22
RN_PrEPOralPlusCon  = 23
RN_PrEPInject1Mo    = 24
RN_PrEPInject2Mo    = 25
RN_PrEPInject6Mo    = 26
RN_PrEPRing         = 27
RN_PrEPbNABs        = 28
RN_PrEPImplant      = 29
RN_PrEP_PEP         = 30
# Medical services for Coverage#
RN_MalesSTITx       = 31
RN_FemalesSTITx     = 32
RN_BloodSafety      = 33
RN_PEP              = 34
RN_ADSyringes       = 35
RN_ReduceInjects    = 36
RN_UniversalPrec    = 37
# Vaccines #
RN_Vaccines         = 38
RN_Cure             = 39
RN_AHDTreatment     = 40
RN_POC_CD4_Int      = 41
RN_POC_VL_Int       = 42

RN_MaxInterventions = 42

RN_PrepInterventions = [i for i in range(RN_PrEPOralDaily, RN_PrEP_PEP + 1)]
RN_PrepInterventions_Male = [i for i in range(RN_PrEPOralDaily, RN_PrEPInject6Mo + 1)] + [RN_PrEPbNABs, RN_PrEPImplant, RN_PrEP_PEP]
RN_PrepInterventions_Female = [i for i in range(RN_PrEPOralDaily, RN_PrEP_PEP + 1)]


#  RN_MaxCovConstants  = 27

# Max coverge constants #
RN_MaxGeneralPopCoverage    = 8
RN_MaxMostAtRiskPopCoverage = 9
RN_MaxMedServicesCoverage   = 7

# Max unit costs constants #
RN_MaxGeneralPopUnitCosts       = 8
RN_MaxMostAtRiskPopUnitCosts    = 9
RN_MaxMedicalServicesUnitCosts  = 5
RN_MaxMaleCircumcisionUnitCosts = 2
RN_MaxCureUnitCosts             = 1
RN_MaxAHDTreatmentUnitCosts     = 1
RN_MaxTestingUnitCosts          = 2
#  RN_MaxPrEPUnitCosts             = 4
RN_MaxVaccinesUnitCosts         = 1
RN_MaxPMTCTUnitCosts            = 14

# Point of care #
RN_POC_CD4 = 1
RN_POC_VL  = 2

# Breakdown of PrEP coverage for CEA interpolation#
RN_CEA_PrEPLRH      = 20
RN_CEA_PrEPMRH      = 21
RN_CEA_PrEPHRH      = 22
RN_CEA_PrEPIDU      = 23
RN_CEA_PrEPMSM      = 24
RN_CEA_ART_Male     = 25
RN_CEA_ART_Female   = 26
RN_MaxCEA           = 26
# Baseline projection for CEA #
RN_BaseLineProj_CEA = 0

# ART unit costs #
RN_CostPerCD4Test              = 1
RN_CostPerViralLoadTest        = 2
RN_CostPerBloodChemistry       = 3
RN_CostPerEIDTest              = 4
RN_CostPerVisit                = 5
RN_CostOfFirstLineARVs         = 6
RN_CostOfSecondLineARVs        = 7
RN_CostOfThirdLineARVs         = 8

# Percent on ART #
RN_PercentPatientsSecondLine   = 1
RN_PercentPatientsThirdLine    = 2

# Test and Visit Schedule #
# Population #
RN_PatientsInitiatingART        = 1
RN_StablePatients               = 2
RN_PatientsNotVirallySuppressed = 3
RN_PregnantWomen                = 4
# Test/Visit #
RN_CD4Test                      = 1
RN_ViralLoadTests               = 2
RN_BloodChemistries             = 3
RN_MedicalConsultations         = 4
RN_DrugDelivAdherenceVisits     = 5
RN_EID                          = 6

# Constants for RNM Summary Tables #
RN_ST_ComMob              = 1
RN_ST_MassMedia           = 2
RN_ST_VCT                 = 3
RN_ST_Condom              = 4
RN_ST_PrimaryTeachers     = 5
RN_ST_SecondaryTeachers   = 6
RN_ST_OutOfSchoolYouth    = 7
RN_ST_CashTransfers       = 8
RN_ST_WorkplaceSTI        = 9
RN_ST_FSW                 = 10
RN_ST_MSW                 = 11
RN_ST_MSMOutreach         = 12
RN_ST_MSMLub              = 13
RN_ST_IDUHarmRed          = 14
RN_ST_IDUCandT            = 15
RN_ST_IDUOutreach         = 16
RN_ST_IDUNSEP             = 17
RN_ST_IDUDrugSub          = 18
RN_ST_MC                  = 19
#  RN_ST_PrEPOral            = 20
#  RN_ST_PrEPInject          = 21
#  RN_ST_PrEPGel             = 22
#  RN_ST_PrEPRing            = 23
RN_ST_PrEPOralDaily       = 20
RN_ST_PrEPOralMonthly     = 21
RN_ST_PrEPOralPlusCon     = 22
RN_ST_PrEPInject1Mo       = 23
RN_ST_PrEPInject2Mo       = 24
RN_ST_PrEPInject6Mo       = 25
RN_ST_PrEPRing            = 26
RN_ST_PrEPbNABs           = 27
RN_ST_PrEPImplant         = 28
RN_ST_PrEP_PEP            = 29
RN_ST_MalesSTITx          = 30
RN_ST_FemalesSTITx        = 31
RN_ST_BloodSafety         = 32
RN_ST_PEP                 = 33
RN_ST_ADSyringes          = 34
RN_ST_ReduceInjects       = 35
RN_ST_UniversalPrec       = 36
RN_ST_Vaccines            = 37
RN_ST_PMTCT               = 38
RN_ST_ARVTherapy          = 39
RN_ST_NonARTCare          = 40
RN_ST_Cure                = 41
RN_ST_AHDTreatment        = 42
RN_ST_POC_CD4_Int         = 43
RN_ST_POC_VL_Int          = 44
RN_ST_MaxSummaryTable     = 44

# Total number of items displayed in Resources Required table (not including
#   the Program Support or user entered pops #
RN_MaxSumTableDisplay    = 28
RN_RowNumBeforeUserPops  = 15  # not sure what this value should be yet


# Sets to exclude intervention constants for specific areas #
RN_NotInCEAorIM = [RN_PrimaryTeachers,RN_WorkplaceSTI, RN_MSW,RN_IDUHarmRed,RN_IDUCandT]

RN_NotInOptimize = [RN_VCT,RN_MSMLub,RN_CEA_PrEPLRH,RN_CEA_PrEPIDU]

RN_NotInCov     = [RN_PrEPOralDaily, RN_PrEPOralMonthly, RN_PrEPOralPlusCon,
RN_PrEPInject1Mo, RN_PrEPInject2Mo, RN_PrEPInject6Mo,
RN_PrEPRing, RN_PrEPbNABs, RN_PrEPImplant, RN_PrEP_PEP, RN_Vaccines]

RN_NumIntervExcludedFromCov = 5

RN_NotInUC    = [RN_FemalesSTITx, RN_ReduceInjects]

RN_NumIntervExcludedFromUC = 2

RN_NotUsedWhenOnlyRNActive = [RN_MCInfant, RN_PrEPOralDaily, RN_PrEPOralMonthly, RN_PrEPOralPlusCon,
RN_PrEPInject1Mo, RN_PrEPInject2Mo, RN_PrEPInject6Mo,
RN_PrEPRing, RN_PrEPbNABs, RN_PrEPImplant, RN_PrEP_PEP, RN_Vaccines, RN_FemalesSTITx, RN_ReduceInjects]
RN_NotUsedWhenGoalsAndRNActive = [RN_MCInfant, RN_FemalesSTITx, RN_ReduceInjects]

# ART Selection constant #
RN_NumOrPercent       = 0
RN_CD4Category        = 1

# Vaccine sheets #
RN_Efficacy          = 1
RN_Infectiousness    = 2
RN_Progression       = 3
RN_Duration          = 4
RN_Type              = 5

RN_Single            = 0
RN_Diff              = 1

RN_OneMSM            = 1
RN_FourMSM           = 4

# Type of vaccine #
RN_TakeAction        = 1
RN_DegreeAction      = 2

#Vaccines Behavioral Effects#
RN_AmongVacc         = 1
RN_AmongAdults       = 2

#Vaccination Status#
RN_AllVacc            = 0
RN_UnV                = 1
RN_Take               = 2
RN_Partial            = 3
RN_NoProt             = 4

# Vaccines Males, PrEP Males and Females #
RN_AllRisk       = 0
RN_None          = 1
RN_LRH           = 2
RN_MRH           = 3
RN_HRH           = 4
RN_IDU           = 5
RN_MSM           = 6
RN_MSMLR         = 7
RN_MSMMR         = 8
RN_MSMHR         = 9
RN_MSMIDU        = 10

# Vaccines and secnario generator Females #
RN_AllRisk_F     = 11
RN_None_F        = 12
RN_LRH_F         = 13
RN_MRH_F         = 14
RN_HRH_F         = 15
RN_IDU_F         = 16
RN_MSM_F         = 17

# Effectiveness Constants #
RN_Effectiveness  = 1
RN_Adherence      = 2
RN_Substitution   = 3
RN_DurationMonths = 4

RN_MethodMixDistanceFromEditor = 60

# Help constants #
RN_Help_Configuration         = 'Configuration_tutorial_RN'
RN_Help_PopSizeGenPop         = 'General_population_Population_Sizes'
RN_Help_PopSizeMARPop         = 'Most_at_risk_populations_Population_Sizes'
RN_Help_PopSizeMedServ        = 'Medical_services_Population_Sizes'
RN_Help_PopSizeMaleCircum     = 'Male_circumcision_Population_Sizes'
RN_Help_CoverageGenPop        = 'General_population_Coverage'
RN_Help_CoverageMARPop        = 'Most_at_risk_populations_Coverage'
RN_Help_CoverageMedServ       = 'Medical_services_Coverage'
RN_Help_CoverageMaleCircum    = 'Male_circumcision_Coverage'
RN_Help_CoverageTreatment     = 'Treatment_Coverage'
RN_Help_UnitCostsGenPop       = 'General_population_Unit_Costs'
RN_Help_UnitCostsMARPop       = 'Most_at_risk_populations_Unit_Costs'
RN_Help_UnitCostsMedServ      = 'Medical_services_Unit_Costs'
RN_Help_UnitCostsMaleCircum   = 'Male_circumcision_Unit_Costs'
RN_Help_UnitCostsPMTCT        = 'PMTCT_Unit_Costs'
RN_Help_UnitCostsTreatment    = 'Treatment_Unit_Costs'
RN_Help_ProgramSupport        = 'Program_support_RNM'
RN_Help_Mitigation            = 'Mitigation_RNM'
RN_Help_CEA                   = 'Cost-effectiveness-and-optimiz'
RN_Help_Optimize              = 'Optimization'

#***** Constants for the Prevention Technologies Scenario tool ******#
RN_PT_DataFileTitle = 'Prevention Technologies scenario file'
RN_PT_Version1      = 1.0

RN_PT_EXT_WILDCARD  = '*.PT'
RN_PT_EXT           = 'PT'
RN_PT_DOTEXT        = '.PT'

#****************   Scenario Generator (SG) constants   ******************#

# .SG file constants #
RN_SG_DataFileTitle = 'Scenario Generator (RNM) Data File'
RN_SG_Version1       = 1.0
RN_SG_Version2       = 2.0
RN_SG_Version3       = 3.0
RN_SG_FileNumSpace  = 20
RN_SG_FileRDec      = 4

RN_SG_EXT_WILDCARD = '*.SGRN'
RN_SG_EXT          = 'SGRN'
RN_SG_DOTEXT       = '.SGRN'

# Constants for the CSV file that the user can read data in from. #
RN_SG_CSVFileVersion = 'RNM Scenario Generator CSV - Version 1'

# Minimum year that the user can enter as the base year in the SG form. #
RN_SG_MinBaseYear = 2008

#   Abbreviations used in the constants below:
#
#       MSM = Men who have sex with men
#       LRH = Low risk heterosexual
#       MRH = Medium risk heterosexual
#       HRH = High risk heterosexual
#       LR = Low risk
#       MR = Medium risk
#       HR = High risk
#       CSRG = Coverage by sex and risk group
#       NSA = Not sexually active
#       IDU = Injecting drug user
#       F = Female
#       M = Male
#

# Intervention current IDs. #
# General population #
RN_SG_ComMob                 = 1
RN_SG_MassMedia              = 2
RN_SG_VCT                    = 3
RN_SG_Condom                 = 4
RN_SG_PrimaryTeachers        = 5
RN_SG_SecondaryTeachers      = 6
RN_SG_OutOfSchoolYouth       = 7
RN_SG_CashTransfers          = 8
RN_SG_WorkplaceSTI           = 9
# Most at risk populations #
RN_SG_FSW                    = 10
RN_SG_MSW                    = 11
RN_SG_MSMOutreach            = 12
RN_SG_MSMLub                 = 13
RN_SG_IDUHarmRed             = 14
RN_SG_IDUCandT               = 15
RN_SG_IDUOutreach            = 16
RN_SG_IDUNSEP                = 17
RN_SG_IDUDrugSub             = 18
# Medical services #
RN_SG_MalesSTITx             = 19
RN_SG_FemalesSTITx           = 20
RN_SG_BloodSafety            = 21
RN_SG_PEP                    = 22
RN_SG_ADSyringes             = 23
RN_SG_ReduceInjects          = 24
RN_SG_UniversalPrec          = 25
# Male circumcision #
RN_SG_MC_15_49               = 26
# PrEP Coverage Male#
RN_SG_PrEP_M_LRH             = 27
RN_SG_PrEP_M_MRH             = 28
RN_SG_PrEP_M_HRH             = 29
RN_SG_PrEP_M_IDU             = 30
RN_SG_PrEP_M_MSM             = 31
RN_SG_PrEP_M_MSM_LR          = 32
RN_SG_PrEP_M_MSM_MR          = 33
RN_SG_PrEP_M_MSM_HR          = 34
RN_SG_PrEP_M_MSM_IDU         = 35
# PrEP Coverage Female#
RN_SG_PrEP_F_LRH             = 36
RN_SG_PrEP_F_MRH             = 37
RN_SG_PrEP_F_HRH             = 38
RN_SG_PrEP_F_IDU             = 39
# PrEP Efficacy #
RN_SG_PrEP_Eff_Oral          = 40
RN_SG_PrEP_Eff_Inject        = 41
RN_SG_PrEP_Eff_Gel           = 42
RN_SG_PrEP_Eff_Ring          = 43
#PrEP Adherence #
RN_SG_PrEP_Adh_Oral          = 44
RN_SG_PrEP_Adh_Inject        = 45
RN_SG_PrEP_Adh_Gel           = 46
RN_SG_PrEP_Adh_Ring          = 47
# Treatment - ART by number or percent #
RN_SG_AdultsNeedRecART_M     = 48
RN_SG_AdultsNeedRecART_F     = 49
RN_SG_MigrLine1To2           = 50
# Treatment - CD4 threshold #
RN_SG_CD4ThreshElig          = 51
# Treatment - Children #
RN_SG_ChildCotrim            = 52
RN_SG_ChildART               = 53
# Treatment - ART by CD4 category #
RN_SG_CD4_GT500              = 54
RN_SG_CD4_350_500            = 55
RN_SG_CD4_250_349            = 56
RN_SG_CD4_200_249            = 57
RN_SG_CD4_100_199            = 58
RN_SG_CD4_50_99              = 59
RN_SG_CD4_LT50               = 60
# Populations eligible for treatment #
RN_SG_EligPregnantWomen      = 61
RN_SG_EligTBHIBCoInfected    = 62
RN_SG_EligDiscordantCouples  = 63
RN_SG_EligSexWorkers         = 64
RN_SG_EligMSM                = 65
RN_SG_EligIDU                = 66
RN_SG_EligOtherPopulation    = 67
# PMTCT #
RN_SG_PMTCT_Pre_Single       = 68
RN_SG_PMTCT_Pre_Dual         = 69
RN_SG_PMTCT_Pre_OptA         = 70
RN_SG_PMTCT_Pre_OptB         = 71
RN_SG_PMTCT_Pre_TripBef      = 72
RN_SG_PMTCT_Pre_TripDur      = 73
RN_SG_PMTCT_Post_OptA        = 74
RN_SG_PMTCT_Post_OptB        = 75
# Vaccines #
RN_SG_CovPercVacc            = 76
# Vaccines by sex and risk group #
RN_SG_CSRG_M_NSA             = 77
RN_SG_CSRG_M_LRH             = 78
RN_SG_CSRG_M_MRH             = 79
RN_SG_CSRG_M_HRH             = 80
RN_SG_CSRG_M_IDU             = 81
RN_SG_CSRG_MSM               = 82
RN_SG_CSRG_MSM_LR            = 83
RN_SG_CSRG_MSM_MR            = 84
RN_SG_CSRG_MSM_HR            = 85
RN_SG_CSRG_MSM_IDU           = 86
RN_SG_CSRG_F_NSA             = 87
RN_SG_CSRG_F_LRH             = 88
RN_SG_CSRG_F_MRH             = 89
RN_SG_CSRG_F_HRH             = 90
RN_SG_CSRG_F_IDU             = 91
# Vaccines - Effectiveness #
RN_SG_ReductSuscept          = 92
RN_SG_ReductInfect           = 93
RN_SG_IncProgression         = 94
RN_SG_VaccineDuration        = 95

RN_SG_MaxInterventions       = 95

# SG intervention current ID sets by common setter methods. #
RN_SG_PopulationsSet  = [RN_SG_ComMob, RN_SG_MassMedia, RN_SG_VCT,
RN_SG_Condom, RN_SG_PrimaryTeachers, RN_SG_SecondaryTeachers,
RN_SG_OutOfSchoolYouth, RN_SG_CashTransfers, RN_SG_WorkplaceSTI,
RN_SG_FSW, RN_SG_MSW, RN_SG_MSMOutreach, RN_SG_MSMLub, RN_SG_IDUHarmRed,
RN_IDUCandT, RN_SG_IDUOutreach, RN_SG_IDUNSEP, RN_SG_IDUDrugSub,
RN_SG_MalesSTITx, RN_SG_FemalesSTITx, RN_SG_BloodSafety, RN_SG_PEP,
RN_SG_ADSyringes, RN_SG_ReduceInjects, RN_SG_UniversalPrec]

RN_SG_PrEPSet_M       = [RN_SG_PrEP_M_LRH, RN_SG_PrEP_M_MRH, RN_SG_PrEP_M_HRH,
RN_SG_PrEP_M_IDU, RN_SG_PrEP_M_MSM, RN_SG_PrEP_M_MSM_LR, RN_SG_PrEP_M_MSM_MR,
RN_SG_PrEP_M_MSM_HR, RN_SG_PrEP_M_MSM_IDU]

RN_SG_PrEPSet_F       = [RN_SG_PrEP_F_LRH, RN_SG_PrEP_F_MRH, RN_SG_PrEP_F_HRH, RN_SG_PrEP_F_IDU]

RN_SG_PrEPSet_Eff     = [RN_SG_PrEP_Eff_Oral, RN_SG_PrEP_Eff_Inject, RN_SG_PrEP_Eff_Gel,
RN_SG_PrEP_Eff_Ring]

RN_SG_PrEPSet_Adh     = [RN_SG_PrEP_Adh_Oral, RN_SG_PrEP_Adh_Inject, RN_SG_PrEP_Adh_Gel,
RN_SG_PrEP_Adh_Ring]

RN_SG_CSRG_Set        = [RN_SG_CSRG_M_NSA, RN_SG_CSRG_M_LRH, RN_SG_CSRG_M_MRH,
RN_SG_CSRG_M_HRH, RN_SG_CSRG_M_IDU, RN_SG_CSRG_MSM, RN_SG_CSRG_MSM_LR,
RN_SG_CSRG_MSM_MR, RN_SG_CSRG_MSM_HR, RN_SG_CSRG_MSM_IDU, RN_SG_CSRG_F_NSA,
RN_SG_CSRG_F_LRH, RN_SG_CSRG_F_MRH, RN_SG_CSRG_F_HRH, RN_SG_CSRG_F_IDU]

RN_SG_ParamSet        = [RN_SG_ReductSuscept, RN_SG_ReductInfect, RN_SG_IncProgression,RN_SG_VaccineDuration]

RN_SG_EligPopSet      = [RN_SG_EligPregnantWomen, RN_SG_EligTBHIBCoInfected, RN_SG_EligDiscordantCouples,
RN_SG_EligSexWorkers, RN_SG_EligMSM, RN_SG_EligIDU, RN_SG_EligOtherPopulation]

RN_SG_CD4Set          = [RN_SG_CD4_GT500, RN_SG_CD4_350_500, RN_SG_CD4_250_349, RN_SG_CD4_200_249,
RN_SG_CD4_100_199, RN_SG_CD4_50_99, RN_SG_CD4_LT50]

RN_SG_PMTCTSet_Pre    = [RN_SG_PMTCT_Pre_Single, RN_SG_PMTCT_Pre_Dual, RN_SG_PMTCT_Pre_OptA, RN_SG_PMTCT_Pre_OptB,
RN_SG_PMTCT_Pre_TripBef, RN_SG_PMTCT_Pre_TripDur]

RN_SG_PMTCTSet_Post   = [RN_SG_PMTCT_Post_OptA, RN_SG_PMTCT_Post_OptB]

RN_SG_VacEffSet       = [RN_SG_ReductSuscept, RN_SG_ReductInfect, RN_SG_IncProgression,RN_SG_VaccineDuration]

# Constants for reading and writing EasyRNMData #
RN_FileYearCol                       = 0
RN_CodeCol                             = 0
RN_CountryCol                           = 1
RN_NumFemaleSWCol                       = 2
RN_NumMaleSWCol                         = 3
RN_NumMSMCol                           = 4
RN_NumIDUCol                           = 5
RN_PercOpioidDepCol                     = 6
RN_InjPerYearCol                       = 7
RN_PrimaryGrEnrRateMaleCol             = 8
RN_PrimaryGrEnrRateFemaleCol           = 9
RN_PrimaryPupTeachRatioCol             = 10
RN_SecondaryGrEnrRateMaleCol           = 11
RN_SecondaryGrEnrRateFemaleCol         = 12
RN_SecondaryPupTeachRatioCol           = 13
RN_FreqTeachRetrainingCol               = 14
RN_FormalSectorEmpWorkProgramsCol       = 15
RN_PercSexuallyActive15_49Col           = 16
RN_PercMalesRegPartnerships15_49Col     = 17
RN_PercMalesNonregPartners15_49Col     = 18
RN_NumSexActsCasualPartnersCol         = 19
RN_NumSexActsRegularPartnersCol         = 20
RN_NumCommSexActsFSWCol                 = 21
RN_NumCommSexActsMSWCol                 = 22
RN_CondomWastageCol                     = 23
RN_NumNewSTIsMaleCol                   = 24
RN_NumNewSTIsFemaleCol                 = 25
RN_PercSymptSTIsMaleCol                 = 26
RN_PercSymptSTIsFemaleCol               = 27
RN_BloodUnitsPer1000Col                 = 28
RN_PEPKitsPerMillionCol                 = 29
RN_AvgNumImmunPerChildCol               = 30
RN_AdultInjectionsCol                   = 31
RN_PercUnsafeInjectionsCol             = 32
RN_HospitalBedsPer1000Col               = 33
RN_RapesPer1000Col                     = 34
RN_CovFSWCol                           = 35
RN_CovMSWCol                           = 36
RN_CovMSMCol                           = 37
RN_CovIDUHarmReductionCol               = 38
RN_CovIDUCandTCol                       = 39
RN_CovIDUOutreachCol                   = 40
RN_CovIDUNSEPCol                       = 41
RN_CovIDUDrugSubCol                     = 42
RN_CovCommMobCol                       = 43
RN_CovPrimStudentsCol                   = 44
RN_CovSecStudentsCol                   = 45
RN_CovOutOfSchoolCol                   = 46
RN_CovCashTransfersCol                 = 47
RN_CovWorkForceSTICol                   = 48
RN_CovCasualCondomsCol                 = 49
RN_CovMSMLubCol                         = 50
RN_CovSTIMalesCol                       = 51
RN_CovSTIFemalesCol                     = 52
RN_CovVCTCol                           = 53
RN_CovMassMediaCol                     = 54
RN_CovUnitsBloodTestedCol               = 55
RN_CovPEPNeedMetCol                     = 56
RN_CovUnsafeInjADSyringesCol           = 57
RN_CovReductionInjCol                   = 58
RN_CovHospitalBedsCol                   = 59
RN_CovMalesCircumcisedCol               = 60
RN_CostFSWCol                           = 61
RN_CostMSWCol                           = 62
RN_CostMSMCol                           = 63
RN_CostIDUHarmRedCol                   = 64
RN_CostIDUCandTCol                     = 65
RN_CostIDUOutreachCol                   = 66
RN_CostIDUNSEPCol                       = 67
RN_CostIDUDrugSubCol                   = 68
RN_CostCommMobCol                       = 69
RN_CostPrimaryCol                       = 70
RN_CostSecondCol                       = 71
RN_CostOutOfSchoolCol                   = 72
RN_CostCashTransfersCol                 = 73
RN_CostSTIWorkplaceCol                 = 74
RN_CostCondomPubCol                     = 75
RN_CostSTICol                           = 76
RN_CostVCTCol                           = 77
RN_CostMassMediaCol                     = 78
RN_CostUnitsBloodTestedCol             = 79
RN_CostPEPKitCol                       = 80
RN_CostADSyringeCol                     = 81
RN_CostHospBedCol                       = 82
RN_CostCircumCol                       = 83
RN_CostCD4Col                        = 84
RN_CostVLCol                         = 85
RN_CostEIDCol                        = 86
RN_CostBloodChemCol                  = 87
RN_CostVisitCol                      = 88
RN_CostFLARVCol                      = 89
RN_CostSLARVCol                      = 90
RN_CostTLARVCol                      = 91
RN_PercSecondLineCol                 = 92
RN_PercThirdLineCol                  = 93
RN_EasyRNMData_MaxCol                = 93

# EasyRNMData year for data in the database #
RN_EasyRNMData_Year = 2016

# String constants for tag column headers. Used for EasyRNMData file. #
RN_CodeTag                           = 'Code'
RN_CountryTag                        = 'Country'
RN_NumFemaleSWTag                    = 'NumFemaleSW'
RN_NumMaleSWTag                      = 'NumMaleSW'
RN_NumMSMTag                         = 'NumMSM'
RN_NumIDUTag                         = 'NumIDU'
RN_PercOpioidDepTag                  = 'PercOpioidDep'
RN_InjPerYearTag                     = 'InjPerYear'
RN_PrimaryGrEnrRateToMaleTag         = 'PrimaryGrEnrRateMale'
RN_PrimaryGrEnrRateToFemaleTag       = 'PrimaryGrEnrRateFemale'
RN_PrimaryPupTeachRatioTag           = 'PrimaryPupTeachRatio'
RN_SecondaryGrEnrRateToMaleTag       = 'SecondaryGrEnrRateMale'
RN_SecondaryGrEnrRateToFemaleTag     = 'SecondaryGrEnrRateFemale'
RN_SecondaryPupTeachRatioTag         = 'SecondaryPupTeachRatio'
RN_FreqTeachRetrainingTag            = 'FreqTeachRetraining'
RN_FormalSectorEmpWorkProgramsTag    = 'FormalSectorEmpWorkPrograms'
RN_15to49PercSexuallyActiveTag       = 'PercSexuallyActive15_49'
RN_15to49MalesPercRegPartnershipsTag = 'PercMalesRegPartnerships15_49'
RN_15to49PercMalesNonregPartnersTag  = 'PercMalesNonregPartners15_49'
RN_NumSexActsCasualPartnersTag       = 'NumSexActsCasualPartners'
RN_NumSexActsRegularPartnersTag      = 'NumSexActsRegularPartners'
RN_NumCommSexActsFSWTag              = 'NumCommSexActsFSW'
RN_NumCommSexActsMSWTag              = 'NumCommSexActsMSW'
RN_CondomWastageTag                  = 'CondomWastage'
RN_NumNewSTIsMaleTag                 = 'NumNewSTIsMale'
RN_NumNewSTIsFemaleTag               = 'NumNewSTIsFemale'
RN_PercSymptSTIsMaleTag              = 'PercSymptSTIsMale'
RN_PercSymptSTIsFemaleTag            = 'PercSymptSTIsFemale'
RN_BloodUnitsPerThousTag             = 'BloodUnitsPer1000'
RN_PEPKitsPerMillionTag              = 'PEPKitsPerMillion'
RN_AvgNumImmunPerChildTag            = 'AvgNumImmunPerChild'
RN_AdultInjectionsTag                = 'AdultInjections'
RN_PercUnsafeInjectionsTag           = 'PercUnsafeInjections'
RN_HospitalBedsPerThousTag           = 'HospitalBedsPer1000'
RN_PercAdultMalesCircumcisedTag      = 'PercAdultMalesCircumcised'
RN_RapesPerThousTag                  = 'RapesPer1000'
RN_CovFSWTag                         = 'CovFSW'
RN_CovMSWTag                         = 'CovMSW'
RN_CovMSMTag                         = 'CovMSM'
RN_CovIDUHarmReductionTag            = 'CovIDUHarmReduction'
RN_CovIDUCAndTTag                    = 'CovIDUCandT'
RN_CovIDUOutreachTag                 = 'CovIDUOutreach'
RN_CovIDUNSEPTag                     = 'CovIDUNSEP'
RN_CovIDUDrugSubTag                  = 'CovIDUDrugSub'
RN_CovCommMobTag                     = 'CovCommMob'
RN_CovPrimStudentsTag                = 'CovPrimStudents'
RN_CovSecStudentsTag                 = 'CovSecStudents'
RN_CovOutOfSchoolTag                 = 'CovOutOfSchool'
RN_CovCashTransfersTag               = 'CovCashTransfers'
RN_CovWorkForceSTITag                = 'CovWorkForceSTI'
RN_CovCasualCondomsTag               = 'CovCasualCondoms'
RN_CovMSMLubTag                      = 'CovMSMLub'
RN_CovSTIMalesTag                    = 'CovSTIMales'
RN_CovSTIFemalesTag                  = 'CovSTIFemales'
RN_CovVCTTag                         = 'CovVCT'
RN_CovMassMediaTag                   = 'CovMassMedia'
RN_CovUnitsBloodTestedTag            = 'CovUnitsBloodTested'
RN_CovPEPNeedMetTag                  = 'CovPEPNeedMet'
RN_CovUnsafeInjADSyringesTag         = 'CovUnsafeInjADSyringes'
RN_PercReductionInjTag               = 'CovReductionInj'
RN_CovHospitalBedsTag                = 'CovHospitalBeds'
RN_CovMalesCircumcisedTag            = 'CovMalesCircumcised'
RN_CostFSWTag                        = 'CostFSW'
RN_CostMSWTag                        = 'CostMSW'
RN_CostMSMTag                        = 'CostMSM'
RN_CostIDUHarmRedTag                 = 'CostIDUHarmRed'
RN_CostIDUCAndTTag                   = 'CostIDUCandT'
RN_CostIDUOutreachTag                = 'CostIDUOutreach'
RN_CostIDUNSEPTTag                   = 'CostIDUNSEP'
RN_CostIDUDrugSubTag                 = 'CostIDUDrugSub'
RN_CostCommMobTag                    = 'CostCommMob'
RN_CostPrimaryTag                    = 'CostPrimary'
RN_CostSecondTag                     = 'CostSecond'
RN_CostOutOfSchoolTag                = 'CostOutOfSchool'
RN_CostCashTransfersTag              = 'CostCashTransfers'
RN_CostSTIWorkplaceTag               = 'CostSTIWorkplace'
RN_CostCondomPubTag                  = 'CostCondomPub'
RN_CostSTITag                        = 'CostSTI'
RN_CostVCTTag                        = 'CostVCT'
RN_CostMassMediaTag                  = 'CostMassMedia'
RN_CostUnitsBloodTestedTag           = 'CostUnitsBloodTested'
RN_CostPEPKitTag                     = 'CostPEPKit'
RN_CostADSyringeTag                  = 'CostADSyringe'
RN_CostHospBedTag                    = 'CostHospBed'
RN_CostCircumTag                     = 'CostCircum'
RN_CostCD4Tag                        = 'CostCD4'
RN_CostVLTag                         = 'CostVL'
RN_CostEIDTag                        = 'CostEID'
RN_CostBloodChemTag                  = 'CostBloodChem'
RN_CostVisitTag                      = 'CostVisit'
RN_CostFLARVTag                      = 'CostFLARV'
RN_CostSLARVTag                      = 'CostSLARV'
RN_CostTLARVTag                      = 'CostTLARV'
RN_PercSecondLineTag                 = 'PercSecondLn'
RN_PercThirdLineTag                  = 'PercThirdLine'

#**** Constants for Impact Express data generator ****#
RN_IE_DataFileTitle = 'Impact Express data generator file'

RN_IE_Version1      = 1.0
RN_IE_FileNumSpace  = 10
RN_IE_FileRDec      = 4

RN_IE_DefaultNumInterations  = '500'
RN_IE_DefaultTargetYear      = '2018'
RN_IE_DefaultFinalYearOutput = '2025'

RN_IE_EXT_WILDCARD = '*.IE'
RN_IE_EXT          = 'IE'
RN_IE_CSV_EXT      = '_IE1.csv'

RN_IE_Delimeter = '-1'

RN_IE_YearIncrement = 1

# Coverages used by the IE tool #
RN_IE_ART_M                   = 1
RN_IE_ART_F                   = 2
RN_IE_ART_Child               = 3
RN_IE_PMTCT                   = 4
RN_IE_TB_CaseDet              = 5
RN_IE_Goals_MC15_49           = 6
RN_IE_Goals_Condoms           = 7
RN_IE_Goals_VCT               = 8
RN_IE_Goals_ComMob            = 9
RN_IE_Goals_SexWorkerOutreach = 10
RN_IE_Goals_MSMOutreach       = 11
RN_IE_Goals_IDUOutreach       = 12
RN_IE_MaxCoverages            = 12

RN_IE_PMTCT_None              = 13
RN_IE_PMTCT_Single            = 14
RN_IE_PMTCT_Dual              = 15
RN_IE_PMTCT_OptA              = 16
RN_IE_PMTCT_OptB              = 17
RN_IE_PMTCT_Before            = 18
RN_IE_PMTCT_During            = 19
RN_IE_NumPMTCTConstants       = 7

RN_IE_TB_Noti                 = 20
RN_IE_TB_Inc                  = 21
RN_IE_NumTBConstants          = 2

# Output constants #
RN_IE_NewInfections_Adult_Out     = 1
RN_IE_NewInfections_Child_Out     = 2
RN_IE_AIDSDeaths_Adult_Out        = 3
RN_IE_AIDSDeaths_Child_Out        = 4
RN_IE_NumReceivingART_Adult_Out   = 5
RN_IE_NumReceivingART_Child_Out   = 6
RN_IE_NumReceivingPMTCT_Out       = 7
RN_IE_Males15_49_Out              = 8
RN_IE_Males15_Out                 = 9
RN_IE_QALYs_Out                   = 10
RN_IE_TB_TotalMortality_Out       = 11
RN_IE_TB_Notification_Out         = 12
RN_IE_Condoms_Out                 = 13
RN_IE_VCT_Out                     = 14
RN_IE_ComMob_Out                  = 15
RN_IE_SexWorkerOutreach_Out       = 16
RN_IE_MSMOutreach_Out             = 17
RN_IE_IDUOutreach_Out             = 18
RN_IE_MaxOutputs                  = 18

# Iteration type #
RN_IE_Random      = 0
RN_IE_Ordered     = 1

# Form that is calling the CEA tool #
RN_ToolsForm      = 1
RN_OptimizeForm   = 2

# For Optimize tool #
RN_OP_ART_Male             = 0
RN_OP_ART                  = 1    #this constant is used for Female and BothSexes (for editor and cost)
RN_OP_VMMC                 = 2
RN_OP_CondomPromotion      = 3
RN_OP_SWOutreach           = 4
RN_OP_MSMOutreach          = 5
RN_OP_PWIDOutreach         = 6
RN_OP_PWIDNSEP             = 7
RN_OP_PWIDOST              = 8
RN_OP_PrEPAdolescents      = 9
RN_OP_PrEPSW               = 10
RN_OP_PrEPMSM              = 11
RN_OP_CommunityMobil       = 12
RN_OP_MediaCommunications  = 13
RN_OP_YouthInSchool        = 14
RN_OP_YouthOutOfSchool     = 15
RN_OP_CashTransfers        = 16
RN_OP_MaxOptimize          = 16

# Optimizer Constants #
RN_MinCov                  = 2
RN_MaxCov                  = 3

# Goal options for Optimize tool #
RN_OP_NewInfections = 0
RN_OP_AIDSDeaths    = 1
RN_OP_DALYs         = 2

# Projections for Optimize tool #
RN_OP_MinimumProj     = '_Minimum'
RN_OP_MaximumProj     = '_Maximum'
RN_OP_OptimizedProj   = '_Optimized'

# RNEdFM Glow button constants #
RN_ConfigBtn          = 1
RN_PopSizesBtn        = 2
RN_CoverageBtn        = 3
RN_TreatmentBtn       = 4
RN_UnitCostsBtn       = 5
RN_ProgramSupportBtn  = 6
RN_MitigationBtn      = 7
RN_ResultsBtn         = 8

# Sheets in RNModData file #
RN_EasyRNMData             = 'EasyRNMData'#'EasyRNMData2016'
RN_EasyRNMOldData          = 'EasyRNMOldData'
RN_PreventionTechnologies  = 'PreventionTechnologies'



from SpectrumCommon.Const.VW.VWConst import *
from SpectrumCommon.Const.DP import *
from SpectrumCommon.Const.AM import *
from SpectrumCommon.Const.FP import *
from SpectrumCommon.Const.CS import *
from AvenirCommon.Util import GBRange

def VW_TotalPop_Init(projection, SubnatData_DB):
    result = np.zeros((VW_Subnat+1)).tolist()
    
    for i in GBRange(VW_National, VW_Subnat):
        result[i] = 0
        for s in GBRange(VW_Male, VW_Female):
            for a in GBRange(0, 80):
                result[i] += projection[DP_BigPopTag][s][a][0]

    if 'PercPop' in SubnatData_DB:
        result[VW_Subnat] *= float(SubnatData_DB['PercPop'])

    return result

def VW_TFR_Init(projection, SubnatData_DB, source):
    result = np.zeros((VW_Subnat+1)).tolist()
    
    for i in GBRange(VW_National, VW_Subnat):
        result[i] = projection[DP_TFRTag][0]

    source = SubnatData_DB[source]
    
    if str(CS_MstTFR) in source:
        record = source[str(CS_MstTFR)]
        if 'value' in record:
            tfr_rate = float(record['value'])
            result[VW_Subnat] = min(tfr_rate * result[VW_National], 100)

    return result

def VW_CPR_Init(projection, SubnatData_DB, source):
    result = np.zeros((VW_Subnat+1)).tolist()
    
    for i in GBRange(VW_National, VW_Subnat):
        result[i] = projection[FP_PrevalenceTag][FP_All_Ages][FP_all_need][0]

    source = SubnatData_DB[source]
    
    if str(CS_MstCPR) in source:
        record = source[str(CS_MstCPR)]
        if 'value' in record:
            cpr_rate = float(record['value'])
            result[VW_Subnat] = min(cpr_rate * result[VW_National], 100)

    return result

def VW_Incidence_Init(projection):
    result = np.zeros((VW_Subnat+1)).tolist()
    
    for i in GBRange(VW_National, VW_Subnat):
        IncidenceOptions = projection[AM_IncidenceOptionsTag]
        result[i] = projection[AM_IncidenceByFitTag][IncidenceOptions][0]

    return result

def VW_PMTCT_Init(projection, t):
    result = np.zeros((VW_Subnat+1, VW_PostnatalProphylaxis+1)).tolist()
    
    ARVRegimen = projection[AM_ARVRegimenTag]
    
    for i in GBRange(VW_National, VW_Subnat):
        
        value = 0
        for r in GBRange(DP_SingleDoseNevir, DP_TripleARTDurPreg_Late):
            value += ARVRegimen[DP_PrenatalProphylaxis][r][DP_Percent][t]
        result[i][VW_PrenatalProphylaxis] = value
        
        value = 0
        for r in GBRange(DP_OptA, DP_OptB):
            value += ARVRegimen[DP_PostnatalProphylaxis][r][DP_Percent][t]
        result[i][VW_PostnatalProphylaxis] = value

    return result

def VW_Cotrim_Init(projection, t):
    result = np.zeros((VW_Subnat+1)).tolist()
    
    for i in GBRange(VW_National, VW_Subnat):
        result[i] = projection[AM_ChildTreatInputsTag][DP_PerChildHIVPosCot][t]

    return result

def VW_AdultART_Init(projection, t):
    result = np.zeros((VW_Subnat+1, VW_Female+1)).tolist()
    
    for i in GBRange(VW_National, VW_Subnat):
        for s in GBRange(VW_Male, VW_Female):
            result[i][s] = projection[AM_HAARTBySexTag][s][t]

    return result


from SpectrumCommon.Const.IH import *
from SpectrumCommon.Const.IC import *

# IH 

IV_IH_NUM_DEFAULT_DELIV_CHANS_COSTED = IH_NUM_DEFAULT_DELIV_CHANS_COSTED

# IC

IV_IC_CS_IMPACT                    = IC_CS_IMPACT                     
IV_IC_AM_IMPACT                    = IC_AM_IMPACT                     
IV_IC_RN_IMPACT                    = IC_RN_IMPACT                     
IV_IC_FP_IMPACT                    = IC_FP_IMPACT                     
IV_IC_TB_IMPACT                    = IC_TB_IMPACT                     
IV_IC_MA_IMPACT                    = IC_MA_IMPACT                     
IV_IC_NC_IMPACT                    = IC_NC_IMPACT                     
IV_IC_IS_PROG_MGMT                 = IC_IS_PROG_MGMT                  
IV_IC_RN_PC_HIV                    = IC_RN_PC_HIV                     
IV_IC_RN_STIS                      = IC_RN_STIS                       
IV_IC_NOT_DELIV_MAIN_CHANNELS      = IC_NOT_DELIV_MAIN_CHANNELS       
IV_IC_BREAST_CANCER_TREATMENT      = IC_BREAST_CANCER_TREATMENT       
IV_IC_TB_NOTIF_TREATMENT           = IC_TB_NOTIF_TREATMENT            
IV_IC_CS_CHILDBIRTH                = IC_CS_CHILDBIRTH                 
IV_IC_PREG_WOMEN_MALARIA           = IC_PREG_WOMEN_MALARIA            
IV_IC_ITN_IRS_MALARIA              = IC_ITN_IRS_MALARIA               
IV_IC_MALARIA_PSEUDO_IMPACT        = IC_MALARIA_PSEUDO_IMPACT         
IV_IC_ART_SET_1                    = IC_ART_SET_1                     
IV_IC_ART_SET_2                    = IC_ART_SET_2                     
IV_IC_PC_MNS                       = IC_PC_MNS                        
IV_IC_NC_CANCER                    = IC_NC_CANCER                     
IV_IC_BASIC_INTENS_DEPRESSION      = IC_BASIC_INTENS_DEPRESSION       
IV_IC_BASIC_INTENS_PSYCHOSIS       = IC_BASIC_INTENS_PSYCHOSIS        
IV_IC_BASIC_INTENS_BIPOLAR         = IC_BASIC_INTENS_BIPOLAR          
IV_IC_BASIC_INTENS_DEV_DISORDERS   = IC_BASIC_INTENS_DEV_DISORDERS    
IV_IC_BASIC_INTENS_BEHAV_DISORDERS = IC_BASIC_INTENS_BEHAV_DISORDERS  
IV_IC_PENTA_VACC                   = IC_PENTA_VACC                    
IV_IC_CS_STANDARD_VACC             = IC_CS_STANDARD_VACC              
IV_IC_CS_DET_VACC                  = IC_CS_DET_VACC                   
IV_IC_DEV_MODE_VACC                = IC_DEV_MODE_VACC                 
IV_IC_BREAST_CANCER_TR_PINS        = IC_BREAST_CANCER_TR_PINS         
IV_IC_WORKERS_HEALTH               = IC_WORKERS_HEALTH                
IV_IC_ADOLESCENT_HEALTH            = IC_ADOLESCENT_HEALTH             
IV_IC_TB_DAT_DISAG                 = IC_TB_DAT_DISAG                  
IV_IC_TB_DAT_AG                    = IC_TB_DAT_AG                     
IV_IC_PHC                          = IC_PHC                           
IV_IC_ALT_COVERAGE                 = IC_ALT_COVERAGE        
IV_IC_HIDE_CONFIG                  = IC_HIDE_CONFIG
IV_IC_HIDE_INPUT_OUTPUT_DATA       = IC_HIDE_INPUT_OUTPUT_DATA
IV_IC_HIDE_INPUT_DATA              = IC_HIDE_INPUT_DATA
IV_IC_HIDE_PINS                    = IC_HIDE_PINS                     
IV_IC_HIDE_TARG_POPS               = IC_HIDE_TARG_POPS                
IV_IC_HIDE_COVERAGES               = IC_HIDE_COVERAGES       
IV_IC_LOCK_COVERAGES               = IC_LOCK_COVERAGES         
IV_IC_HIDE_TREAT_INPUTS            = IC_HIDE_TREAT_INPUTS             
IV_IC_HIDE_DELIV_CHANNELS          = IC_HIDE_DELIV_CHANNELS           
IV_IC_HIDE_RESULTS                 = IC_HIDE_RESULTS                  
IV_IC_CALC_PINS_ALL_YEARS_LOCKED   = IC_CALC_PINS_ALL_YEARS_LOCKED    
IV_IC_CALC_PINS_ALL_YEARS_OVERRIDE = IC_CALC_PINS_ALL_YEARS_OVERRIDE  
IV_IC_CALC_PINS_TWO_PLUS_LOCKED    = IC_CALC_PINS_TWO_PLUS_LOCKED     
IV_IC_MULTIPLY_PINS_100            = IC_MULTIPLY_PINS_100             
IV_IC_PINS_OVER_100                = IC_PINS_OVER_100                 
IV_IC_MALARIA_PINS                 = IC_MALARIA_PINS                  

IV_IC_WASH_CHAN_CURR_ID             = IC_WASH_CHAN_CURR_ID
IV_IC_OTHER_NON_HEALTH_CHAN_CURR_ID = IC_OTHER_NON_HEALTH_CHAN_CURR_ID
IV_IC_PRIVATE_SECTOR_CHAN_CURR_ID   = IC_PRIVATE_SECTOR_CHAN_CURR_ID
IV_IC_NUM_DELIV_CHANS               = IC_NUM_DELIV_CHANS

IV_IC_NUM_YEAR_ENDPOINTS = IC_NUM_YEAR_ENDPOINTS


from SpectrumCommon.Modvars.GB.GBDefs import *
from SpectrumCommon.Const.DP import *

from SpectrumCommon.Modvars.DP.DPIndices import *

sourceDictShape = {
    'Name' : 'str',
    'Summary' : 'str',
    'Source' : 'str',
    'Date' : 'str',
}

DP_modvar_shapes = {
    DP_DataSourceTag : gb_str_shape('DP_DataSourceTag'),
    # DP_DPSourcesTag : gb_shape('UPD Sources', type='dict', 
    #     innerShape={
    #         'Provider' : 'str',
    #         'VersionNum' : 'int',
    #         'Sources' : gb_shape('Data type', type='1d array of dict', indices=dp_upd_source_indices, 
    #             innerShape=gb_shape('Source type', type='dict',
    #                 innerShape=sourceDictShape
    #             )
    #         )
    #     }),
    DP_DefaultUPDLETag : gb_sex_year_shape('DP_DefaultUPDLETag'),
    DP_DefaultUPDIMRTag : gb_sex_year_shape('DP_DefaultUPDIMRTag'),
    DP_DefaultUPDCMRTag : gb_sex_year_shape('DP_DefaultUPDCMRTag'),
    DP_DefaultUPDSurvRateTag : gb_shape('DP_DefaultUPDSurvRateTag', type='3D float',
        dim=['surv_rate_ages', gb_sex_dim, gb_year_dim], 
        indices=[dp_surv_rate_indices, gb_sex_indices, gb_year_indices]),
    DP_TFRTag : gb_year_shape('Total fertility rate'),
    DP_ASFRTag : gb_5y_age_year_shape('DP_ASFRTag'),
    DP_UNPopASFRTag : gb_5y_age_year_shape('DP_UNPopASFRTag'),
    DP_ASFRTablesTag : gb_shape('DP_ASFRTablesTag', type='3D float',
        dim=['tables', 'cols', 'rows'], 
        indices=[dp_asfr_table_indices, dp_asfr_col_indices, dp_asfr_row_indices], has_default=False),
    DP_ASFRNumTag : gb_int_shape('DP_ASFRNumTag'),
    DP_ASFRCustomFlagTag : gb_bool_shape('DP_ASFRCustomFlagTag'),
    DP_SexBirthRatioTag : gb_year_shape('DP_SexBirthRatioTag'),
    DP_LETag : gb_sex_year_shape('Life expectancy'),
    DP_MigrRateTag : gb_sex_5y_age_year_shape('DP_MigrRateTag'),
    DP_MigrAgeDistTag : gb_sex_5y_age_year_shape('DP_MigrAgeDistTag'),
    DP_ModelLifeFileNameTag : gb_str_shape('DP_ModelLifeFileNameTag', has_default=False),
    DP_UseFYrSingleAgesTag : gb_bool_shape('DP_UseFYrSingleAgesTag', has_default=False),
    DP_RegionalAdjustPopDataTag : gb_sex_single_age_year_shape('DP_RegionalAdjustPopDataTag', has_default=False),
    DP_RegionalAdjustPopCBStateTag : gb_bool_shape('DP_RegionalAdjustPopCBStateTag', has_default=False),
    DP_RegionalAdjustPopFNameTag : gb_str_shape('DP_RegionalAdjustPopFNameTag',has_default=False),
    DP_RegionalAdjustPopFDateTag : gb_str_shape('DP_RegionalAdjustPopFDateTag', has_default=False),
    # DP_CustomFileYearsTag : gb_shape('DP_CustomFileYearsTag', type='dict',
    #     innerShape = {
    #         'DP_CPopFirstYr' : 'int',
    #         'DP_CPopFinalYr' : 'int'
    #     },
    #      has_default=False),
    DP_CustomPopStopRescalingYearTag : gb_int_shape('DP_CustomPopStopRescalingYearTag', has_default=False),
    DP_LifeTableNumTag : gb_int_shape('DP_LifeTableNumTag'),
    DP_SurvRateTag : gb_shape('DP_SurvRateTag', type='3D float',
        dim=['surv_rate_ages', gb_sex_dim, gb_year_dim], 
        indices=[dp_surv_rate_indices, gb_sex_indices, gb_year_indices]),
    
    DP_BigPopTag : gb_sex_single_age_year_shape('DP_BigPopTag'),
    DP_InitialConditionsTag : gb_shape('DP_InitialConditionsTag', type='dict', 
        innerShape={
            'pop1'      : gb_shape('pop1', '4D float',
                dim=[gb_sex_dim, gb_single_age_dim, 'hiv', 'dur'], 
                indices=[{'male' : 0, 'female' : 1}, 
                         gb_single_age_indices, 
                         {('hiv ' + str(a)) : a for a in GBRange(0, 7)},
                         {('dur ' + str(a)) : a for a in GBRange(0, 7)}
                ]
            ),
            'births'    : gb_shape('births', '2D float',
                dim=[gb_sex_dim, gb_single_age_dim], 
                indices=[{'male' : 0, 'female' : 1}, gb_single_age_indices]),
            'deaths'    : gb_shape('deaths', '2D float',
                dim=[gb_sex_dim, gb_single_age_dim], 
                indices=[{'male' : 0, 'female' : 1}, gb_single_age_indices])
        }),
    
    ##outputs## : gb_shape('##outputs##', type='incomplete', has_default=False),
    DP_BirthsTag : gb_sex_5y_age_year_shape('DP_BirthsTag', has_default=False),
    DP_DeathsByAgeTag : gb_sex_single_age_year_shape('DP_DeathsByAgeTag', has_default=False),
    DP_IMRTag : gb_sex_year_shape('DP_IMRTag'),
    DP_CMRTag : gb_sex_year_shape('DP_CMRTag'),
    DP_AdjLETag : gb_sex_year_shape('DP_AdjLETag', has_default=False),
    
    #DPTempModvars : gb_shape('#DPTempModvars', type='incomplete', has_default=False),
    
    DP_ResultAgeSpecificMortalityTag : gb_sex_single_age_year_shape('DP_ResultAgeSpecificMortalityTag', has_default=False),
    DP_ResultDeathsByAgeLastBirthdayTag : gb_sex_single_age_year_shape('DP_ResultDeathsByAgeLastBirthdayTag', has_default=False),
    DP_ResultCWRTag : gb_year_shape('DP_ResultCWRTag', has_default=False),
    DP_ResultGRRTag : gb_year_shape('DP_ResultGRRTag', has_default=False),
    DP_ResultMACTag : gb_year_shape('DP_ResultMACTag', has_default=False),
    DP_ResultNRRTag : gb_year_shape('DP_ResultNRRTag', has_default=False),
    DP_ResultIMRTag : gb_sex_year_shape('DP_ResultIMRTag', has_default=False),
    DP_ResultU5MRTag : gb_sex_year_shape('DP_ResultU5MRTag', has_default=False),
    DP_Result45q15Tag : gb_year_shape('DP_Result45q15Tag', has_default=False),
    DP_ResultCBRTag : gb_year_shape('DP_ResultCBRTag', has_default=False),
    DP_ResultCDRTag : gb_year_shape('DP_ResultCDRTag', has_default=False),
    DP_ResultDoublingTimeTag : gb_year_shape('DP_ResultDoublingTimeTag', has_default=False),
    DP_ResultGRTag : gb_year_shape('DP_ResultGRTag', has_default=False),
    DP_ResultRNITag : gb_year_shape('DP_ResultRNITag', has_default=False),
    DP_ResultMedianAgeTag : gb_sex_year_shape('DP_ResultMedianAgeTag', has_default=False),
    
    DP_UALowHighRatiosTag : gb_shape('DP_UALowHighRatiosTag', type='dictList',
        innerShape = {
            'isValidUAData' : 'bool',
            'mstID' : 'int',
            'values' : gb_shape('values', '3D Float', 
                dim=[gb_sex_dim, 'high/low ratios', gb_single_age_dim], 
                indices=[gb_sex_dim, {'dead space' : 0, 'low ratio' : DPUA_LowRatio, 'high ratio' : DPUA_HighRatio}, gb_year_indices]
            )
        }, 
        has_default=False),
    DP_AIDSDeathsFYearUATag : gb_int_shape('DP_AIDSDeathsFYearUATag', has_default=False),
    DP_UAHasRanTag : gb_bool_shape('DP_UAHasRanTag', has_default=False),
    # DP_UAInfoTag : gb_shape('DP_UAInfoTag', type='dict',
    #     innerShape = {
    #         'ProcessDate' : 'str',
    #         'ProcessDateDelphi' : 'int',
    #         'ProcessTime' : 'str',
    #         'NumIterations' : 'int',
    #         'AgDataYear' : 'int'
    #     }, 
    #     has_default=False),
}